@@ -1,5 +1,7 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Stefan Beller
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -22,31 +24,31 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  */
 
 #include "CampaignManager.h"
+#include "MenuItemStorage.h"
+#include "Settings.h"
+#include "StatBlock.h"
 #include "SharedResources.h"
+#include "UtilsParsing.h"
+#include "LootManager.h"
 
-CampaignManager::CampaignManager() {
+#include <sstream>
 
-	drop_stack.item = 0;
-	drop_stack.quantity = 0;
+using namespace std;
 
-	items = NULL;
-	carried_items = NULL;
-	currency = NULL;
-	xp = NULL;
-	
-	log_msg = "";
-	quest_update = true;
-	
-	clearAll();
-
-}
+CampaignManager::CampaignManager()
+	: status()
+	, log_msg("")
+	, drop_stack()
+	, items(NULL)
+	, carried_items(NULL)
+	, currency(NULL)
+	, hero(NULL)
+	, quest_update(true)
+{}
 
 void CampaignManager::clearAll() {
 	// clear campaign data
-	for (int i=0; i<MAX_STATUS; i++) {
-		status[i] = "";
-	}
-	status_count = 0;
+	status.clear();
 }
 
 /**
@@ -55,7 +57,7 @@ void CampaignManager::clearAll() {
 void CampaignManager::setAll(std::string s) {
 	string str = s + ',';
 	string token;
-	while (str != "" && status_count < MAX_STATUS) {
+	while (str != "") {
 		token = eatFirstString(str, ',');
 		if (token != "") this->setStatus(token);
 	}
@@ -68,9 +70,9 @@ void CampaignManager::setAll(std::string s) {
 std::string CampaignManager::getAll() {
 	stringstream ss;
 	ss.str("");
-	for (int i=0; i<status_count; i++) {
+	for (unsigned int i=0; i < status.size(); i++) {
 		ss << status[i];
-		if (i < status_count-1) ss << ',';
+		if (i < status.size()-1) ss << ',';
 	}
 	return ss.str();
 }
@@ -79,8 +81,8 @@ bool CampaignManager::checkStatus(std::string s) {
 
 	// avoid searching empty statuses
 	if (s == "") return false;
-	
-	for (int i=0; i<status_count; i++) {
+
+	for (unsigned int i=0; i < status.size(); i++) {
 		if (status[i] == s) return true;
 	}
 	return false;
@@ -91,15 +93,12 @@ void CampaignManager::setStatus(std::string s) {
 	// avoid adding empty statuses
 	if (s == "") return;
 
-	// hit upper limit for status
-	// TODO: add a warning
-	if (status_count >= MAX_STATUS) return;
-	
 	// if it's already set, don't add it again
 	if (checkStatus(s)) return;
-	
-	status[status_count++] = s;
+
+	status.push_back(s);
 	quest_update = true;
+	hero->check_title = true;
 }
 
 void CampaignManager::unsetStatus(std::string s) {
@@ -107,17 +106,16 @@ void CampaignManager::unsetStatus(std::string s) {
 	// avoid searching empty statuses
 	if (s == "") return;
 
-	for (int i=status_count-1; i>=0; i--) {
-		if (status[i] == s) {
-		
-			// bubble existing statuses down
-			for (int j=i; j<status_count-1; j++) {
-				status[j] = status[j+1];
-			}
-			status_count--;
+	vector<string>::iterator it;
+	// see http://stackoverflow.com/a/223405
+	for (it = status.end(); it != status.begin();) {
+		--it;
+		if ((*it) == s) {
+			it = status.erase(it);
 			quest_update = true;
 			return;
 		}
+		hero->check_title = true;
 	}
 }
 
@@ -131,7 +129,7 @@ void CampaignManager::removeItem(int item_id) {
 
 void CampaignManager::rewardItem(ItemStack istack) {
 
-	if (carried_items->full()) {
+	if (carried_items->full(istack.item)) {
 		drop_stack.item = istack.item;
 		drop_stack.quantity = istack.quantity;
 	}
@@ -142,23 +140,50 @@ void CampaignManager::rewardItem(ItemStack istack) {
 			addMsg(msg->get("You receive %s.", items->items[istack.item].name));
 		if (istack.quantity > 1)
 			addMsg(msg->get("You receive %s x%d.", istack.quantity, items->items[istack.item].name));
-		
+
 		items->playSound(istack.item);
 	}
 }
 
 void CampaignManager::rewardCurrency(int amount) {
 	*currency += amount;
-	addMsg(msg->get("You receive %d gold.", amount));
-	items->playCoinsSound();
+	addMsg(msg->get("You receive %d %s.", amount, CURRENCY));
+	LootManager::getInstance()->playCurrencySound();
 }
 
-void CampaignManager::rewardXP(int amount) {
-	*xp += amount;
-	addMsg(msg->get("You receive %d XP.", amount));
-}	
+void CampaignManager::rewardXP(int amount, bool show_message) {
+	hero->xp += (amount * (100 + hero->effects.bonus_xp)) / 100;
+	hero->refresh_stats = true;
+	if (show_message) addMsg(msg->get("You receive %d XP.", amount));
+}
 
-void CampaignManager::addMsg(string new_msg) {
+void CampaignManager::restoreHPMP(std::string s) {
+	if (s == "hp") {
+		hero->hp = hero->maxhp;
+		addMsg(msg->get("HP restored."));
+	}
+	else if (s == "mp") {
+		hero->mp = hero->maxmp;
+		addMsg(msg->get("MP restored."));
+	}
+	else if (s == "hpmp") {
+		hero->hp = hero->maxhp;
+		hero->mp = hero->maxmp;
+		addMsg(msg->get("HP and MP restored."));
+	}
+	else if (s == "status") {
+		hero->effects.clearNegativeEffects();
+		addMsg(msg->get("Negative effects removed."));
+	}
+	else if (s == "all") {
+		hero->hp = hero->maxhp;
+		hero->mp = hero->maxmp;
+		hero->effects.clearNegativeEffects();
+		addMsg(msg->get("HP and MP restored, negative effects removed"));
+	}
+}
+
+void CampaignManager::addMsg(const string& new_msg) {
 	if (log_msg != "") log_msg += " ";
 	log_msg += new_msg;
 }
