@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger and kitano
+Copyright © 2011-2012 Clint Bellanger and kitano
 
 This file is part of FLARE.
 
@@ -22,23 +22,25 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * This base class handles logic common to all of these child classes
  */
 
+
+#pragma once
 #ifndef ENTITY_H
 #define ENTITY_H
 
-#include "MapIso.h"
-#include "Animation.h"
-#include "Utils.h"
+#include "StatBlock.h"
 #include <vector>
 
+class Animation;
+class AnimationSet;
+class MapRenderer;
+
 class Entity {
 protected:
 	SDL_Surface *sprites;
-	Animation *activeAnimation;
-	MapIso* map;
-	vector<Animation*> animations;
 
 public:
-	Entity(MapIso*);
+	Entity(MapRenderer*);
+	Entity(const Entity&);
 	virtual ~Entity();
 
 	bool move();
@@ -47,10 +49,11 @@ public:
 	// Each child of Entity defines its own rendering method
 	virtual Renderable getRender() = 0;
 
-	void loadAnimations(const std::string& filename);
-
 	bool setAnimation(const std::string& animation);
+	Animation *activeAnimation;
+	AnimationSet *animationSet;
 
+	MapRenderer* map;
 	StatBlock stats;
 };
 
