@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -16,16 +16,24 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 */
 
 #include "FileParser.h"
+#include "UtilsParsing.h"
 
-FileParser::FileParser() {
-	line = "";
-	section = "";
-	key = "";
-	val = "";
-}
 
-bool FileParser::open(string filename) {
-	
+using namespace std;
+	bool new_section;
+	std::string section;
+	std::string key;
+	std::string val;
+
+FileParser::FileParser()
+	: line("")
+	, new_section(false)
+	, section("")
+	, key("")
+	, val("")
+{}
+
+bool FileParser::open(const string& filename) {
 	infile.open(filename.c_str(), ios::in);
 	return infile.is_open();
 }
@@ -45,8 +53,8 @@ bool FileParser::next() {
 
 	string starts_with;
 	new_section = false;
-	
-	while (!infile.eof()) {
+
+	while (infile.good()) {
 
 		line = getLine(infile);
 
@@ -54,25 +62,25 @@ bool FileParser::next() {
 		if (line.length() == 0) continue;
 
 		starts_with = line.at(0);
-		
+
 		// skip ahead if this line is a comment
 		if (starts_with == "#") continue;
-		
+
 		// set new section if this line is a section declaration
 		if (starts_with == "[") {
 			new_section = true;
 			section = parse_section_title(line);
-			
+
 			// keep searching for a key-pair
 			continue;
 		}
-		
+
 		// this is a keypair. Perform basic parsing and return
 		parse_key_pair(line, key, val);
 		return true;
-		
+
 	}
-	
+
 	// hit the end of file
 	return false;
 }
@@ -82,8 +90,8 @@ bool FileParser::next() {
  */
 string FileParser::getRawLine() {
 	line = "";
-	
-	if (!infile.eof()) {
+
+	if (infile.good()) {
 		line = getLine(infile);
 	}
 	return line;
@@ -95,13 +103,17 @@ string FileParser::nextValue() {
 	}
 	string s;
 	size_t seppos = val.find_first_of(',');
+	size_t alt_seppos = val.find_first_of(';');
+	if (alt_seppos != string::npos && alt_seppos < seppos)
+	    seppos = alt_seppos; // return the first ',' or ';'
+
 	if (seppos == string::npos) {
 		s = val;
 		val = "";
 	}
 	else {
 		s = val.substr(0, seppos);
-		val = val.substr(seppos+1, val.length());
+		val = val.substr(seppos+1);
 	}
 	return s;
 }
