@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -21,33 +21,33 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Abstract the generic key-value pair ini-style file format
  */
 
+
+#pragma once
 #ifndef FILE_PARSER_H
 #define FILE_PARSER_H
 
 #include <fstream>
 #include <string>
-#include "UtilsParsing.h"
 
 class FileParser {
 private:
-	ifstream infile;
-	string line;
-	
+	std::ifstream infile;
+	std::string line;
+
 public:
 	FileParser();
 	~FileParser();
-	
-	bool open(string filename);
+
+	bool open(const std::string& filename);
 	void close();
 	bool next();
-	string nextValue();
-	string getRawLine();
+	std::string nextValue(); // next value inside one line.
+	std::string getRawLine();
 
 	bool new_section;
-	string section;
-	string key;
-	string val;
-	
+	std::string section;
+	std::string key;
+	std::string val;
 };
 
 #endif
