@@ -1,5 +1,5 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
 
 This file is part of FLARE.
 
@@ -15,6 +15,8 @@ You should have received a copy of the GNU General Public License along with
 FLARE.  If not, see http://www.gnu.org/licenses/
 */
 
+
+#pragma once
 #ifndef GAMESTATE_H
 #define GAMESTATE_H
 
@@ -25,17 +27,19 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 class GameState {
 public:
 	GameState();
+	virtual ~GameState();
 
 	virtual void logic();
 	virtual void render();
 
 	GameState* getRequestedGameState();
-	bool isExitRequested() { return exitRequested; };
+	bool isExitRequested() { return exitRequested; }
 	bool hasMusic;
+	bool reload_music;
 
 protected:
 
-	GameState* requestedGameState;	
+	GameState* requestedGameState;
 
 	bool exitRequested;
 };
