@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -17,27 +17,21 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 
 /**
  * GameStateLoad
- * 
+ *
  * Display the current save-game slots
  * Allow the player to continue a previous game
  * Allow the player to start a new game
  * Allow the player to abandon a previous game
  */
 
+
+#pragma once
 #ifndef GAMESTATELOAD_H
 #define GAMESTATELOAD_H
 
-#include "Settings.h"
-#include "InputState.h"
-#include "FontEngine.h"
-#include "WidgetButton.h"
-#include "FileParser.h"
-#include "Settings.h"
 #include "StatBlock.h"
-#include "ItemManager.h"
 #include "GameState.h"
-#include "MenuConfirm.h"
-#include "SharedResources.h"
+#include "WidgetLabel.h"
 
 #include <SDL.h>
 #include <SDL_image.h>
@@ -46,7 +40,9 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include <string>
 #include <sstream>
 
-
+class ItemManager;
+class MenuConfirm;
+class WidgetButton;
 class WidgetLabel;
 
 
@@ -57,14 +53,17 @@ private:
 
 	void loadGraphics();
 	void loadPortrait(int slot);
-	string getMapName(string map_filename);
+	std::string getMapName(const std::string& map_filename);
+	void updateButtons();
 
 	ItemManager *items;
 	WidgetButton *button_exit;
 	WidgetButton *button_action;
 	WidgetButton *button_alternate;
 	WidgetLabel *label_loading;
-	WidgetLabel *label_slots;
+	WidgetLabel *label_name[GAME_SLOT_MAX];
+	WidgetLabel *label_level[GAME_SLOT_MAX];
+	WidgetLabel *label_map[GAME_SLOT_MAX];
 
 	MenuConfirm *confirm;
 
@@ -72,36 +71,44 @@ private:
 	SDL_Surface *selection;
 	SDL_Surface *portrait_border;
 	SDL_Surface *portrait;
-	SDL_Surface *sprites[GAME_SLOT_MAX];
+	std::vector<SDL_Surface *> sprites[GAME_SLOT_MAX];
 	StatBlock stats[GAME_SLOT_MAX];
-	int equipped[GAME_SLOT_MAX][3];	
+	std::vector<int> equipped[GAME_SLOT_MAX];
+	std::vector<std::string> preview_layer;
 	SDL_Rect slot_pos[GAME_SLOT_MAX];
-	string current_map[GAME_SLOT_MAX];
+	std::string current_map[GAME_SLOT_MAX];
 
 	bool loading_requested;
 	bool loading;
 	bool loaded;
-	
-	Point name_pos;
-	Point level_pos;
-	Point map_pos;
+
+	LabelInfo name_pos;
+	LabelInfo level_pos;
+	LabelInfo map_pos;
+	LabelInfo loading_pos;
 	Point sprites_pos;
-	
+
 	// animation info
 	int current_frame;
 	int frame_ticker;
-	
+
+	SDL_Rect portrait_pos;
+	SDL_Rect gameslot_pos;
+	SDL_Rect preview_pos;
+
+	SDL_Color color_normal;
+
 public:
 	GameStateLoad();
 	~GameStateLoad();
 
 	void logic();
 	void logicLoading();
-	void render();	
+	void render();
 	void readGameSlot(int slot);
 	void readGameSlots();
 	void loadPreview(int slot);
-	
+
 	bool load_game;
 	int selected_slot;
 };
