@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Igor Paliychuk
 
 This file is part of FLARE.
 
@@ -22,48 +23,71 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Also handles message passing between child objects, often to avoid circular dependencies.
  */
 
+
+#pragma once
 #ifndef GAMESTATEPLAY_H
 #define GAMESTATEPLAY_H
 
-#include "InputState.h"
-#include "Avatar.h"
-#include "Enemy.h"
-#include "MapIso.h"
-#include "Utils.h"
-#include "HazardManager.h"
-#include "EnemyManager.h"
-#include "FontEngine.h"
-#include "MenuManager.h"
-#include "LootManager.h"
-#include "PowerManager.h"
-#include "NPCManager.h"
-#include "CampaignManager.h"
-#include "QuestLog.h"
 #include "GameState.h"
-#include "WidgetLabel.h"
 
 #include <SDL.h>
 #include <SDL_image.h>
 
+#include <string>
+#include <vector>
+
+class Avatar;
+class CampaignManager;
+class Enemy;
+class EnemyManager;
+class HazardManager;
+class ItemManager;
+class LootManager;
+class MapRenderer;
+class MenuManager;
+class NPCManager;
+class PowerManager;
+class QuestLog;
+class WidgetLabel;
+
+class Title{
+public:
+	std::string title;
+	int level;
+	int power;
+	std::string requires_status;
+	std::string requires_not;
+	std::string primary_stat;
+
+	Title()
+	 : title("")
+	 , level(0)
+	 , power(0)
+	 , requires_status("")
+	 , requires_not("")
+	 , primary_stat("")
+	{}
+};
+
 class GameStatePlay : public GameState {
 private:
-
-	MapIso *map;
 	Enemy *enemy;
-	Renderable r[1024];
-	int renderableCount;
-	HazardManager *hazards;
+
+	PowerManager *powers;
+	ItemManager *items;
+	CampaignManager *camp;
+	MapRenderer *map;
+	Avatar *pc;
 	EnemyManager *enemies;
+	HazardManager *hazards;
 	MenuManager *menu;
 	LootManager *loot;
-	PowerManager *powers;
 	NPCManager *npcs;
-	CampaignManager *camp;
 	QuestLog *quests;
 
-	WidgetLabel *label_mapname;
-	WidgetLabel *label_fps;
-	
+	WidgetLabel *loading;
+	SDL_Surface *loading_bg;
+
 	bool restrictPowerUse();
 	void checkEnemyFocus();
 	void checkLoot();
@@ -72,12 +96,22 @@ private:
 	void checkCancel();
 	void checkLog();
 	void checkEquipmentChange();
+	void checkTitle();
 	void checkConsumable();
 	void checkNotifications();
 	void checkNPCInteraction();
+	void checkStash();
+	void showLoading();
+	void loadTitles();
 
 	int npc_id;
-	
+	bool eventDialogOngoing;
+	bool eventPendingDialog;
+
+	SDL_Color color_normal;
+
+	std::vector<Title> titles;
+
 public:
 	GameStatePlay();
 	~GameStatePlay();
@@ -87,10 +121,12 @@ public:
 	void showFPS(int fps);
 	void saveGame();
 	void loadGame();
+	void loadClass(int index);
+	void loadStash();
 	void resetGame();
-
-	Avatar *pc;
 	int game_slot;
+
+	Avatar *getAvatar() const { return pc; }
 };
 
 #endif
