@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -15,32 +15,35 @@ You should have received a copy of the GNU General Public License along with
 FLARE.  If not, see http://www.gnu.org/licenses/
 */
 
+
+#pragma once
 #ifndef GAMESTATETITLE_H
 #define GAMESTATETITLE_H
 
-#include "Settings.h"
 #include "GameState.h"
-#include "WidgetButton.h"
-#include "WidgetLabel.h"
+
+class WidgetButton;
+class WidgetLabel;
 
 class GameStateTitle : public GameState {
 private:
 	SDL_Surface *logo;
 	WidgetButton *button_play;
 	WidgetButton *button_exit;
+    WidgetButton *button_cfg;
 	WidgetLabel *label_version;
-	
+
 public:
 	GameStateTitle();
 	~GameStateTitle();
 	void loadGraphics();
 	void logic();
 	void render();
-	
+
 	// switch
 	bool exit_game;
 	bool load_game;
-	
+
 };
 
 #endif
