@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Igor Paliychuk
 
 This file is part of FLARE.
 
@@ -15,6 +16,25 @@ You should have received a copy of the GNU General Public License along with
 FLARE.  If not, see http://www.gnu.org/licenses/
 */
 
+
+#pragma once
+#ifndef GAME_SWITCHER_H
+#define GAME_SWITCHER_H
+
+#include <SDL.h>
+#include <SDL_image.h>
+#include <SDL_mixer.h>
+
+#include "Utils.h"
+#include "WidgetLabel.h"
+
+const int GAME_STATE_TITLE = 0;
+const int GAME_STATE_PLAY = 1;
+const int GAME_STATE_LOAD = 2;
+const int GAME_STATE_NEW = 3;
+
+class GameState;
+
 /**
  * class GameSwitcher
  *
@@ -27,37 +47,28 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * - load game screen
  * - maybe full-video cutscenes
  */
- 
-#ifndef GAME_SWITCHER_H
-#define GAME_SWITCHER_H
-
-#include <SDL.h>
-#include <SDL_image.h>
-#include <SDL_mixer.h>
-#include "Settings.h"
-#include "InputState.h"
-#include "FontEngine.h"
-
-const int GAME_STATE_TITLE = 0;
-const int GAME_STATE_PLAY = 1;
-const int GAME_STATE_LOAD = 2;
-const int GAME_STATE_NEW = 3;
-
-class GameState;
 
 class GameSwitcher {
 private:
 	Mix_Music *music;
-	
+
 	GameState *currentState;
-	
+
+	WidgetLabel *label_fps;
+	Point fps_position;
+	SDL_Color fps_color;
+	std::string fps_corner;
+
 public:
 	GameSwitcher();
+	GameSwitcher(const GameSwitcher &copy); // not implemented.
+	~GameSwitcher();
+
 	void loadMusic();
+	void loadFPS();
 	void logic();
 	void render();
-	~GameSwitcher();
-	
+	void showFPS(int fps);
 	bool done;
 };
 
