@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -22,6 +22,8 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * These are generated whenever something makes any attack
  */
 
+
+#pragma once
 #ifndef HAZARD_H
 #define HAZARD_H
 
@@ -32,8 +34,10 @@ class Entity;
 #include <SDL_image.h>
 #include <SDL_mixer.h>
 #include "Utils.h"
-#include "MapCollision.h"
-#include "StatBlock.h"
+
+class Animation;
+class StatBlock;
+class MapCollision;
 
 // the spell/power's source type: eg. which team did it come from?
 const int SOURCE_TYPE_HERO = 0;
@@ -42,29 +46,35 @@ const int SOURCE_TYPE_ENEMY = 2;
 
 class Hazard {
 private:
-	MapCollision *collider;
+	const MapCollision *collider;
 	// Keeps track of entities already hit
 	std::vector<Entity*> entitiesCollided;
-      
+	Animation *activeAnimation;
+	std::string animation_name;
+
 public:
-	Hazard();
+	Hazard(MapCollision *_collider);
+
+	Hazard(const Hazard&); // not implemented! If you implement this, take care to create a real copy of the pointers, such as Animation.
+	Hazard & operator= (const Hazard & other); // same as copy constructor!
+
+	~Hazard();
 
 	StatBlock *src_stats;
 
-	SDL_Surface *sprites;
-	void setCollision(MapCollision *_collider);
 	void logic();
-	
+
 	bool hasEntity(Entity*);
-	
 	void addEntity(Entity*);
 
+	void loadAnimation(std::string &s);
+
 	int dmg_min;
 	int dmg_max;
 	int crit_chance;
 	int accuracy;
 	int source_type;
-	
+
 	FPoint pos;
 	FPoint speed;
 	int base_speed;
@@ -72,47 +82,39 @@ public:
 	int radius;
 	int power_index;
 
-	// visualization info
-	bool rendered;
-	Point frame_size;
-	Point frame_offset;
-	int frame;
-	int frame_loop;
-	int frame_duration;
-	int active_frame; // some hazards are only dangerous on a single frame of their existence
-	int direction; // some hazard animations are 8-directional
-	int visual_option; // some hazard animations have random/varietal options
+	int animationKind;	// direction or other, it is a specific value according to
+						// some hazard animations are 8-directional
+						// some hazard animations have random/varietal options
+
+	bool isDangerousNow();
+	void addRenderable(std::vector<Renderable> &r, std::vector<Renderable> &r_dead);
+
 	bool floor; // rendererable goes on the floor layer
 	int delay_frames;
 	bool complete_animation; // if not multitarget but hitting a creature, still complete the animation?
-	
+
 	// these work in conjunction
 	// if the attack is not multitarget, set active=false
 	// only process active hazards for collision
 	bool multitarget;
 	bool active;
-	
+
 	bool remove_now;
 	bool hit_wall;
-	
+
 	// after effects of various powers
-	int stun_duration;
-	int immobilize_duration;
-	int slow_duration;
-	int bleed_duration;
 	int hp_steal;
 	int mp_steal;
-	
+
 	bool trait_armor_penetration;
 	int trait_crits_impaired;
 	int trait_elemental;
-	
+
 	// pre/post power effects
 	int post_power;
 	int wall_power;
-	
-	bool equipment_modified;
-	
+	int mod_power;
+
 };
 
 #endif
