@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -20,22 +20,26 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  *
  * Holds the collection of hazards (active attacks, spells, etc) and handles group operations
  */
- 
+
+
+#pragma once
 #ifndef HAZARD_MANAGER_H
 #define HAZARD_MANAGER_H
 
-#include "Avatar.h"
-#include "EnemyManager.h"
 #include "Utils.h"
-#include "Hazard.h"
-#include "MapCollision.h"
-#include "PowerManager.h"
+
+#include <vector>
+
+class Avatar;
+class EnemyManager;
+class Hazard;
+class MapCollision;
+class PowerManager;
 
 class HazardManager {
 private:
 	Avatar *hero;
 	EnemyManager *enemies;
-	MapCollision *collider;
 	PowerManager *powers;
 public:
 	HazardManager(PowerManager *_powers, Avatar *_hero, EnemyManager *_enemies);
@@ -43,11 +47,10 @@ public:
 	void logic();
 	void expire(int index);
 	void checkNewHazards();
-	void handleNewMap(MapCollision *_collider);
-	Renderable getRender(int haz_id);
-	
-	int hazard_count;
-	Hazard *h[256];
+	void handleNewMap();
+	void addRenders(std::vector<Renderable> &r, std::vector<Renderable> &r_dead);
+
+	std::vector<Hazard*> h;
 };
 
 #endif
