@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Stefan Beller
 
 This file is part of FLARE.
 
@@ -21,29 +22,59 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Handles keyboard and mouse states
  */
 
+#include <iostream>
+
+#include "FileParser.h"
 #include "InputState.h"
 #include "Settings.h"
+#include "SharedResources.h"
+#include "UtilsDebug.h"
+#include "UtilsParsing.h"
 
 using namespace std;
 
-InputState::InputState(void) {
-
+InputState::InputState(void)
+	: mx_vel(0)
+	, my_vel(0)
+	, done(false)
+	, mouse()
+	, last_key(0)
+	, last_button(0)
+	, scroll_up(false)
+	, scroll_down(false)
+	, mouse_emulation(false)
+{
 	SDL_EnableUNICODE(true);
 
+	defaultQwertyKeyBindings();
+	defaultJoystickBindings();
+
+	for (int key=0; key<key_count; key++) {
+		pressing[key] = false;
+		lock[key] = false;
+	}
+
+	loadKeyBindings();
+	setKeybindNames();
+}
+
+
+void InputState::defaultQwertyKeyBindings ()
+{
 	binding[CANCEL] = SDLK_ESCAPE;
 	binding[ACCEPT] = SDLK_RETURN;
 	binding[UP] = SDLK_w;
 	binding[DOWN] = SDLK_s;
 	binding[LEFT] = SDLK_a;
 	binding[RIGHT] = SDLK_d;
-	
+
 	binding_alt[CANCEL] = SDLK_ESCAPE;
 	binding_alt[ACCEPT] = SDLK_SPACE;
 	binding_alt[UP] = SDLK_UP;
 	binding_alt[DOWN] = SDLK_DOWN;
 	binding_alt[LEFT] = SDLK_LEFT;
 	binding_alt[RIGHT] = SDLK_RIGHT;
-	
+
 	binding[BAR_1] = binding_alt[BAR_1] = SDLK_1;
 	binding[BAR_2] = binding_alt[BAR_2] = SDLK_2;
 	binding[BAR_3] = binding_alt[BAR_3] = SDLK_3;
@@ -54,35 +85,38 @@ InputState::InputState(void) {
 	binding[BAR_8] = binding_alt[BAR_8] = SDLK_8;
 	binding[BAR_9] = binding_alt[BAR_9] = SDLK_9;
 	binding[BAR_0] = binding_alt[BAR_0] = SDLK_0;
-	
+
 	binding[CHARACTER] = binding_alt[CHARACTER] = SDLK_c;
 	binding[INVENTORY] = binding_alt[INVENTORY] = SDLK_i;
 	binding[POWERS] = binding_alt[POWERS] = SDLK_p;
 	binding[LOG] = binding_alt[LOG] = SDLK_l;
-	
+
 	binding[MAIN1] = binding_alt[MAIN1] = SDL_BUTTON_LEFT;
 	binding[MAIN2] = binding_alt[MAIN2] = SDL_BUTTON_RIGHT;
-	
+
 	binding[CTRL] = SDLK_LCTRL;
 	binding_alt[CTRL] = SDLK_RCTRL;
 	binding[SHIFT] = SDLK_LSHIFT;
 	binding_alt[SHIFT] = SDLK_RSHIFT;
 	binding[DEL] = SDLK_DELETE;
 	binding_alt[DEL] = SDLK_BACKSPACE;
-	
+}
+
+void InputState::defaultJoystickBindings ()
+{
+	// most joystick buttons are unbound by default
 	for (int key=0; key<key_count; key++) {
-		pressing[key] = false;
-		lock[key] = false;
-	}
-	done = false;
-	
-	loadKeyBindings();
-	
-	// Optionally ignore the Joystick subsystem
-	if (!ENABLE_JOYSTICK) {
-		SDL_JoystickEventState(SDL_IGNORE);
-		SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
+		binding_joy[key] = -1;
 	}
+
+	binding_joy[MAIN1] = 0;
+	binding_joy[MAIN2] = 1;
+	binding_joy[ACCEPT] = 2;
+	binding_joy[CANCEL] = 3;
+	binding_joy[CHARACTER] = 4;
+	binding_joy[INVENTORY] = 5;
+	binding_joy[LOG] = 6;
+	binding_joy[POWERS] = 7;
 }
 
 /**
@@ -90,23 +124,25 @@ InputState::InputState(void) {
  */
 void InputState::loadKeyBindings() {
 
-	FileParser infile;	
+	FileParser infile;
 	int key1;
 	int key2;
 	int cursor;
 
-	if (!infile.open(PATH_CONF + "keybindings.txt")) {
-		saveKeyBindings();
-		return;
+	if (!infile.open(PATH_CONF + FILE_KEYBINDINGS)) {
+		if (!infile.open(mods->locate("engine/default_keybindings.txt").c_str())) {
+			saveKeyBindings();
+			return;
+		} else saveKeyBindings();
 	}
-	
+
 	while (infile.next()) {
 
 		key1 = eatFirstInt(infile.val, ',');
-		key2 = atoi(infile.val.c_str());
-		
+		key2 = toInt(infile.val);
+
 		cursor = -1;
-		
+
 		if (infile.key == "cancel") cursor = CANCEL;
 		else if (infile.key == "accept") cursor = ACCEPT;
 		else if (infile.key == "up") cursor = UP;
@@ -132,7 +168,7 @@ void InputState::loadKeyBindings() {
 		else if (infile.key == "ctrl") cursor = CTRL;
 		else if (infile.key == "shift") cursor = SHIFT;
 		else if (infile.key == "delete") cursor = DEL;
-		
+
 		if (cursor != -1) {
 			binding[cursor] = key1;
 			binding_alt[cursor] = key2;
@@ -147,10 +183,10 @@ void InputState::loadKeyBindings() {
  */
 void InputState::saveKeyBindings() {
 	ofstream outfile;
-	outfile.open((PATH_CONF + "keybindings.txt").c_str(), ios::out);
+	outfile.open((PATH_CONF + FILE_KEYBINDINGS).c_str(), ios::out);
 
 	if (outfile.is_open()) {
-	
+
 		outfile << "cancel=" << binding[CANCEL] << "," << binding_alt[CANCEL] << "\n";
 		outfile << "accept=" << binding[ACCEPT] << "," << binding_alt[ACCEPT] << "\n";
 		outfile << "up=" << binding[UP] << "," << binding_alt[UP] << "\n";
@@ -169,20 +205,22 @@ void InputState::saveKeyBindings() {
 		outfile << "bar0=" << binding[BAR_0] << "," << binding_alt[BAR_0] << "\n";
 		outfile << "main1=" << binding[MAIN1] << "," << binding_alt[MAIN1] << "\n";
 		outfile << "main2=" << binding[MAIN2] << "," << binding_alt[MAIN2] << "\n";
-		outfile << "charater=" << binding[CHARACTER] << "," << binding_alt[CHARACTER] << "\n";
+		outfile << "character=" << binding[CHARACTER] << "," << binding_alt[CHARACTER] << "\n";
 		outfile << "inventory=" << binding[INVENTORY] << "," << binding_alt[INVENTORY] << "\n";
 		outfile << "powers=" << binding[POWERS] << "," << binding_alt[POWERS] << "\n";
 		outfile << "log=" << binding[LOG] << "," << binding_alt[LOG] << "\n";
 		outfile << "ctrl=" << binding[CTRL] << "," << binding_alt[CTRL] << "\n";
 		outfile << "shift=" << binding[SHIFT] << "," << binding_alt[SHIFT] << "\n";
 		outfile << "delete=" << binding[DEL] << "," << binding_alt[DEL] << "\n";
-		
+
+		if (outfile.bad()) fprintf(stderr, "Unable to write keybindings config file. No write access or disk is full!\n");
 		outfile.close();
+		outfile.clear();
 	}
 
 }
 
-void InputState::handle() {
+void InputState::handle(bool dump_event) {
 	SDL_Event event;
 
 	SDL_GetMouseState(&mouse.x, &mouse.y);
@@ -193,12 +231,18 @@ void InputState::handle() {
 	static bool joyHasMovedY;
 	static int joyLastPosX;
 	static int joyLastPosY;
+	int joyAxisXval;
+	int joyAxisYval;
 
 	inkeys = "";
 
 	/* Check for events */
 	while (SDL_PollEvent (&event)) {
 
+		if (dump_event) {
+			cout << event << endl;
+		}
+
 		// grab symbol keys
 		if (event.type == SDL_KEYDOWN) {
 			int ch = event.key.keysym.unicode;
@@ -218,9 +262,15 @@ void InputState::handle() {
 		switch (event.type) {
 
 			case SDL_MOUSEBUTTONDOWN:
-				for (int key=0; key<key_count; key++) {
-					if (event.button.button == binding[key] || event.button.button == binding_alt[key]) {
-						pressing[key] = true;
+				if (event.button.button == SDL_BUTTON_WHEELUP) {
+					scroll_up = true;
+				} else if (event.button.button == SDL_BUTTON_WHEELDOWN) {
+					scroll_down = true;
+				}else {
+					for (int key=0; key<key_count; key++) {
+						if (event.button.button == binding[key] || event.button.button == binding_alt[key]) {
+							pressing[key] = true;
+						}
 					}
 				}
 				break;
@@ -231,6 +281,7 @@ void InputState::handle() {
 						lock[key] = false;
 					}
 				}
+				last_button = event.button.button;
 				break;
 			case SDL_KEYDOWN:
 				for (int key=0; key<key_count; key++) {
@@ -246,185 +297,15 @@ void InputState::handle() {
 						lock[key] = false;
 					}
 				}
+				last_key = event.key.keysym.sym;
 				break;
+			/*
 			case SDL_JOYAXISMOTION:
-				if(JOYSTICK_DEVICE == event.jaxis.which)
-				{
-					switch(event.jaxis.axis) {
-						/* first analog */
-						case 0:
-							if(event.jaxis.value < -JOY_DEADZONE)
-							{
-								if(!joyReverseAxisX)
-								{
-									if(joyLastPosX == JOY_POS_RIGHT)
-									{
-										joyHasMovedX = 0;
-									}
-								}
-								else
-								{
-									if(joyLastPosX == JOY_POS_LEFT)
-									{
-										joyHasMovedX = 0;
-									}
-								}
-								if(joyHasMovedX == 0)
-								{
-									if(!joyReverseAxisX)
-									{
-										pressing[LEFT] = true;
-										pressing[RIGHT] = false;
-										lock[RIGHT] = false;
-										joyLastPosX = JOY_POS_LEFT;
-									}
-									else
-									{
-										pressing[RIGHT] = true;
-										pressing[LEFT] = false;
-										lock[LEFT] = false;
-										joyLastPosX = JOY_POS_RIGHT;
-									}
-									joyHasMovedX = 1;
-								}
-							}
-							if(event.jaxis.value > JOY_DEADZONE)
-			                                {
-								if(!joyReverseAxisX)
-								{
-									if(joyLastPosX == JOY_POS_LEFT)
-									{
-										joyHasMovedX = 0;
-									}
-								}
-								else
-								{
-									if(joyLastPosX == JOY_POS_RIGHT)
-									{
-										joyHasMovedX = 0;
-									}
-								}
-								if(joyHasMovedX == 0)
-								{
-									if(!joyReverseAxisX)
-									{
-										pressing[RIGHT] = true;
-										pressing[LEFT] = false;
-										lock[LEFT] = false;
-										joyLastPosX = JOY_POS_RIGHT;
-									}
-									else
-									{
-										pressing[LEFT] = true;
-										pressing[RIGHT] = false;
-										lock[RIGHT] = false;
-										joyLastPosX = JOY_POS_LEFT;
-									}
-									joyHasMovedX = 1;
-								}
-			                                }
-							if((event.jaxis.value >= -JOY_DEADZONE) && (event.jaxis.value < JOY_DEADZONE))
-							{
-								pressing[LEFT] = false;
-								lock[LEFT] = false;
-								pressing[RIGHT] = false;
-								lock[RIGHT] = false;
-								joyHasMovedX = 0;
-								joyLastPosX = JOY_POS_CENTER;
-							}
-							break;
-						case 1:
-							if(event.jaxis.value < -JOY_DEADZONE)
-			                                {
-								if(!joyReverseAxisY)
-								{
-									if(joyLastPosY == JOY_POS_DOWN)
-									{
-										joyHasMovedY = 0;
-									}
-								}
-								else
-								{
-									if(joyLastPosY == JOY_POS_UP)
-									{
-										joyHasMovedY = 0;
-									}
-								}
-								if(joyHasMovedY == 0)
-								{
-									if(!joyReverseAxisY)
-									{
-										pressing[UP] = true;
-										pressing[DOWN] = false;
-										lock[DOWN] = false;
-										joyLastPosY = JOY_POS_UP;
-									}
-									else
-									{
-										pressing[DOWN] = true;
-										pressing[UP] = false;
-										lock[UP] = false;
-										joyLastPosY = JOY_POS_DOWN;
-									}
-									joyHasMovedY = 1;
-								}
-			                                }
-							if(event.jaxis.value > JOY_DEADZONE)
-			                                {
-								if(!joyReverseAxisY)
-								{
-									if(joyLastPosY == JOY_POS_UP)
-									{
-										joyHasMovedY = 0;
-									}
-								}
-								else
-								{
-									if(joyLastPosY == JOY_POS_DOWN)
-									{
-										joyHasMovedY = 0;
-									}
-								}
-								if(joyHasMovedY == 0)
-								{
-									if(!joyReverseAxisY)
-									{
-										pressing[DOWN] = true;
-										pressing[UP] = false;
-										lock[UP] = false;
-										joyLastPosY = JOY_POS_DOWN;
-									}
-									else
-									{
-										pressing[UP] = true;
-										pressing[DOWN] = false;
-										lock[DOWN] = false;
-										joyLastPosY = JOY_POS_UP;
-									}
-									joyHasMovedY = 1;
-								}
-							}
-							if((event.jaxis.value >= -JOY_DEADZONE) && (event.jaxis.value < JOY_DEADZONE))
-							{
-								pressing[UP] = false;
-								lock[UP] = false;
-								pressing[DOWN] = false;
-								lock[DOWN] = false;
-								joyHasMovedY = 0;
-								joyLastPosY = JOY_POS_CENTER;
-							}
-							break;
-						/* second analog */
-						case 2:
-							break;
-						case 4:
-							break;
-					}
-				}
+				// Reading joystick from SDL_JOYAXISMOTION is slow. Joystick analog input is handled by SDL_JoystickGetAxis() now.
 				break;
-
+			*/
 			case SDL_JOYHATMOTION:
-				if(JOYSTICK_DEVICE == event.jhat.which)
+				if(JOYSTICK_DEVICE == event.jhat.which && ENABLE_JOYSTICK)
 				{
 					switch (event.jhat.value) {
 						case SDL_HAT_CENTERED:
@@ -509,20 +390,20 @@ void InputState::handle() {
 				}
 				break;
 			case SDL_JOYBUTTONDOWN:
-				if(JOYSTICK_DEVICE == event.jbutton.which)
+				if(JOYSTICK_DEVICE == event.jbutton.which && ENABLE_JOYSTICK)
 				{
 					for (int key=0; key<key_count; key++) {
-						if (event.jbutton.button == binding[key] || event.jbutton.button == binding_alt[key]) {
+						if (event.jbutton.button == binding_joy[key]) {
 							pressing[key] = true;
 						}
 					}
 				}
 				break;
 			case SDL_JOYBUTTONUP:
-				if(JOYSTICK_DEVICE == event.jbutton.which)
+				if(JOYSTICK_DEVICE == event.jbutton.which && ENABLE_JOYSTICK)
 				{
 					for (int key=0; key<key_count; key++) {
-						if (event.jbutton.button == binding[key] || event.jbutton.button == binding_alt[key]) {
+						if (event.jbutton.button == binding_joy[key]) {
 							pressing[key] = false;
 							lock[key] = false;
 						}
@@ -537,7 +418,291 @@ void InputState::handle() {
 		}
 	}
 
-		
+	// joystick analog input
+	if(ENABLE_JOYSTICK)
+	{
+		joyAxisXval = SDL_JoystickGetAxis(joy, 0);
+		joyAxisYval = SDL_JoystickGetAxis(joy, 1);
+
+		// axis 0
+		if(joyAxisXval < -JOY_DEADZONE)
+		{
+			if(!joyReverseAxisX)
+			{
+				if(joyLastPosX == JOY_POS_RIGHT)
+				{
+					joyHasMovedX = 0;
+				}
+			}
+			else
+			{
+				if(joyLastPosX == JOY_POS_LEFT)
+				{
+					joyHasMovedX = 0;
+				}
+			}
+			if(joyHasMovedX == 0)
+			{
+				if(!joyReverseAxisX)
+				{
+					pressing[LEFT] = true;
+					pressing[RIGHT] = false;
+					lock[RIGHT] = false;
+					joyLastPosX = JOY_POS_LEFT;
+				}
+				else
+				{
+					pressing[RIGHT] = true;
+					pressing[LEFT] = false;
+					lock[LEFT] = false;
+					joyLastPosX = JOY_POS_RIGHT;
+				}
+				joyHasMovedX = 1;
+			}
+		}
+		if(joyAxisXval > JOY_DEADZONE)
+		{
+			if(!joyReverseAxisX)
+			{
+				if(joyLastPosX == JOY_POS_LEFT)
+				{
+					joyHasMovedX = 0;
+				}
+			}
+			else
+			{
+				if(joyLastPosX == JOY_POS_RIGHT)
+				{
+					joyHasMovedX = 0;
+				}
+			}
+			if(joyHasMovedX == 0)
+			{
+				if(!joyReverseAxisX)
+				{
+					pressing[RIGHT] = true;
+					pressing[LEFT] = false;
+					lock[LEFT] = false;
+					joyLastPosX = JOY_POS_RIGHT;
+				}
+				else
+				{
+					pressing[LEFT] = true;
+					pressing[RIGHT] = false;
+					lock[RIGHT] = false;
+					joyLastPosX = JOY_POS_LEFT;
+				}
+				joyHasMovedX = 1;
+			}
+		}
+		if((joyAxisXval >= -JOY_DEADZONE) && (joyAxisXval < JOY_DEADZONE))
+		{
+			pressing[LEFT] = false;
+			lock[LEFT] = false;
+			pressing[RIGHT] = false;
+			lock[RIGHT] = false;
+			joyHasMovedX = 0;
+			joyLastPosX = JOY_POS_CENTER;
+		}
+
+		// axis 1
+		if(joyAxisYval < -JOY_DEADZONE)
+		{
+			if(!joyReverseAxisY)
+			{
+				if(joyLastPosY == JOY_POS_DOWN)
+				{
+					joyHasMovedY = 0;
+				}
+			}
+			else
+			{
+				if(joyLastPosY == JOY_POS_UP)
+				{
+					joyHasMovedY = 0;
+				}
+			}
+			if(joyHasMovedY == 0)
+			{
+				if(!joyReverseAxisY)
+				{
+					pressing[UP] = true;
+					pressing[DOWN] = false;
+					lock[DOWN] = false;
+					joyLastPosY = JOY_POS_UP;
+				}
+				else
+				{
+					pressing[DOWN] = true;
+					pressing[UP] = false;
+					lock[UP] = false;
+					joyLastPosY = JOY_POS_DOWN;
+				}
+				joyHasMovedY = 1;
+			}
+		}
+		if(joyAxisYval > JOY_DEADZONE)
+		{
+			if(!joyReverseAxisY)
+			{
+				if(joyLastPosY == JOY_POS_UP)
+				{
+					joyHasMovedY = 0;
+				}
+			}
+			else
+			{
+				if(joyLastPosY == JOY_POS_DOWN)
+				{
+					joyHasMovedY = 0;
+				}
+			}
+			if(joyHasMovedY == 0)
+			{
+				if(!joyReverseAxisY)
+				{
+					pressing[DOWN] = true;
+					pressing[UP] = false;
+					lock[UP] = false;
+					joyLastPosY = JOY_POS_DOWN;
+				}
+				else
+				{
+					pressing[UP] = true;
+					pressing[DOWN] = false;
+					lock[DOWN] = false;
+					joyLastPosY = JOY_POS_UP;
+				}
+				joyHasMovedY = 1;
+			}
+		}
+		if((joyAxisYval >= -JOY_DEADZONE) && (joyAxisYval < JOY_DEADZONE))
+		{
+			pressing[UP] = false;
+			lock[UP] = false;
+			pressing[DOWN] = false;
+			lock[DOWN] = false;
+			joyHasMovedY = 0;
+			joyLastPosY = JOY_POS_CENTER;
+		}
+
+		mouseEmulation();
+	}
+}
+
+void InputState::resetScroll() {
+	scroll_up = false;
+	scroll_down = false;
+}
+
+void InputState::enableMouseEmulation() {
+	if (ENABLE_JOYSTICK && !mouse_emulation) {
+		mouse_emulation = true;
+		SDL_WarpMouse(VIEW_W_HALF,VIEW_H_HALF);
+	}
+}
+
+void InputState::disableMouseEmulation() {
+	if (ENABLE_JOYSTICK && mouse_emulation) {
+		mouse_emulation = false;
+		SDL_WarpMouse(VIEW_W-1,VIEW_H-1);
+	}
+}
+
+void InputState::mouseEmulation() {
+	if (!mouse_emulation) return;
+
+	if (pressing[UP] && my_vel > -MOUSE_EMU_VEL) my_vel--;
+	else if (!pressing[UP] && my_vel < 0) my_vel = 0;
+
+	if (pressing[DOWN] && my_vel < MOUSE_EMU_VEL) my_vel++;
+	else if (!pressing[DOWN] && my_vel > 0) my_vel = 0;
+
+	if (pressing[LEFT] && mx_vel > -MOUSE_EMU_VEL) mx_vel--;
+	else if (!pressing[LEFT] && mx_vel < 0) mx_vel = 0;
+
+	if (pressing[RIGHT] && mx_vel < MOUSE_EMU_VEL) mx_vel++;
+	else if (!pressing[RIGHT] && mx_vel > 0) mx_vel = 0;
+
+	if (mx_vel != 0 || my_vel != 0) SDL_WarpMouse(mouse.x+mx_vel,mouse.y+my_vel);
+}
+
+void InputState::lockActionBar() {
+	pressing[BAR_1] = false;
+	pressing[BAR_2] = false;
+	pressing[BAR_3] = false;
+	pressing[BAR_4] = false;
+	pressing[BAR_5] = false;
+	pressing[BAR_6] = false;
+	pressing[BAR_7] = false;
+	pressing[BAR_8] = false;
+	pressing[BAR_9] = false;
+	pressing[BAR_0] = false;
+	pressing[MAIN1] = false;
+	pressing[MAIN2] = false;
+	lock[BAR_1] = true;
+	lock[BAR_2] = true;
+	lock[BAR_3] = true;
+	lock[BAR_4] = true;
+	lock[BAR_5] = true;
+	lock[BAR_6] = true;
+	lock[BAR_7] = true;
+	lock[BAR_8] = true;
+	lock[BAR_9] = true;
+	lock[BAR_0] = true;
+	lock[MAIN1] = true;
+	lock[MAIN2] = true;
+}
+
+void InputState::unlockActionBar() {
+	lock[BAR_1] = false;
+	lock[BAR_2] = false;
+	lock[BAR_3] = false;
+	lock[BAR_4] = false;
+	lock[BAR_5] = false;
+	lock[BAR_6] = false;
+	lock[BAR_7] = false;
+	lock[BAR_8] = false;
+	lock[BAR_9] = false;
+	lock[BAR_0] = false;
+	lock[MAIN1] = false;
+	lock[MAIN2] = false;
+}
+
+void InputState::setKeybindNames() {
+	binding_name[0] = msg->get("Cancel");
+	binding_name[1] = msg->get("Accept");
+	binding_name[2] = msg->get("Up");
+	binding_name[3] = msg->get("Down");
+	binding_name[4] = msg->get("Left");
+	binding_name[5] = msg->get("Right");
+	binding_name[6] = msg->get("Bar1");
+	binding_name[7] = msg->get("Bar2");
+	binding_name[8] = msg->get("Bar3");
+	binding_name[9] = msg->get("Bar4");
+	binding_name[10] = msg->get("Bar5");
+	binding_name[11] = msg->get("Bar6");
+	binding_name[12] = msg->get("Bar7");
+	binding_name[13] = msg->get("Bar8");
+	binding_name[14] = msg->get("Bar9");
+	binding_name[15] = msg->get("Bar0");
+	binding_name[16] = msg->get("Character");
+	binding_name[17] = msg->get("Inventory");
+	binding_name[18] = msg->get("Powers");
+	binding_name[19] = msg->get("Log");
+	binding_name[20] = msg->get("Main1");
+	binding_name[21] = msg->get("Main2");
+	binding_name[22] = msg->get("Ctrl");
+	binding_name[23] = msg->get("Shift");
+	binding_name[24] = msg->get("Delete");
+
+	mouse_button[0] = msg->get("lmb");
+	mouse_button[1] = msg->get("mmb");
+	mouse_button[2] = msg->get("rmb");
+	mouse_button[3] = msg->get("wheel up");
+	mouse_button[4] = msg->get("wheel down");
+	mouse_button[5] = msg->get("mbx1");
+	mouse_button[6] = msg->get("mbx2");
 }
 
 InputState::~InputState() {
