@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -19,7 +19,14 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class ItemStorage
  */
 
+#include "ItemManager.h"
 #include "ItemStorage.h"
+#include "UtilsParsing.h"
+
+#include <sstream>
+
+using namespace std;
+
 
 void ItemStorage::init(int _slot_number, ItemManager *_items) {
 	slot_number = _slot_number;
@@ -44,8 +51,22 @@ void ItemStorage::setItems(string s) {
 	s = s + ',';
 	for (int i=0; i<slot_number; i++) {
 		storage[i].item = eatFirstInt(s, ',');
-		if( storage[i].item != 0) storage[i].quantity = 1;
-		else storage[i].quantity = 0;
+		// check if such item exists to avoid crash if savegame was modified manually
+		if (storage[i].item < 0) {
+			fprintf(stderr, "Item on position %d has negative id, skipping\n", i);
+			storage[i].item = 0;
+			storage[i].quantity = 0;
+		}
+		else if ((unsigned)storage[i].item > items->items.size()-1) {
+			fprintf(stderr, "Item id (%d) out of bounds 1-%d, skipping\n", storage[i].item, (int)items->items.size());
+			storage[i].item = 0;
+			storage[i].quantity = 0;
+		}
+		else if (storage[i].item != 0 && items->items[storage[i].item].name == "") {
+			fprintf(stderr, "Item with id=%d. found on position %d does not exist, skipping\n", storage[i].item, i);
+			storage[i].item = 0;
+			storage[i].quantity = 0;
+		}
 	}
 }
 
@@ -56,9 +77,17 @@ void ItemStorage::setQuantities(string s) {
 	s = s + ',';
 	for (int i=0; i<slot_number; i++) {
 		storage[i].quantity = eatFirstInt(s, ',');
+		if (storage[i].quantity < 0) {
+			fprintf(stderr, "Items quantity on position %d is negative, setting to zero\n", i);
+			storage[i].quantity = 0;
+		}
 	}
 }
 
+int ItemStorage::getSlotNumber() {
+	return slot_number;
+}
+
 /**
  * Convert storage array to a CSV list of items id for savefile
  */
@@ -99,22 +128,19 @@ void ItemStorage::clear() {
  * @param slot Slot number where it will try to store the item
  */
 void ItemStorage::add( ItemStack stack, int slot) {
-	int max_quantity;
-	int quantity_added;
-	int i;
 
-	if( stack.item != 0) {
-		max_quantity = items->items[stack.item].max_quantity;
-		if( slot > -1) {
+	if (stack.item != 0) {
+		int max_quantity = items->items[stack.item].max_quantity;
+		if (slot > -1) {
 			// a slot is specified
-			if( storage[slot].item != 0 && storage[slot].item != stack.item) {
+			if (storage[slot].item != 0 && storage[slot].item != stack.item) {
 				// the proposed slot isn't available
 				slot = -1;
 			}
 		} else {
 			// first search of stack to complete if the item is stackable
-			i = 0;
-			while( max_quantity > 1 && slot == -1 && i < slot_number) {
+			int i = 0;
+			while (max_quantity > 1 && slot == -1 && i < slot_number) {
 				if (storage[i].item == stack.item && storage[i].quantity < max_quantity) {
 					slot = i;
 				}
@@ -122,27 +148,26 @@ void ItemStorage::add( ItemStack stack, int slot) {
 			}
 			// then an empty slot
 			i = 0;
-			while( slot == -1 && i < slot_number) {
+			while (slot == -1 && i < slot_number) {
 				if (storage[i].item == 0) {
 					slot = i;
 				}
 				i++;
 			}
 		}
-		if( slot != -1) {
+		if (slot != -1) {
 			// Add
-			quantity_added = min( stack.quantity, max_quantity - storage[slot].quantity);
+			int quantity_added = min( stack.quantity, max_quantity - storage[slot].quantity);
 			storage[slot].item = stack.item;
 			storage[slot].quantity += quantity_added;
 			stack.quantity -= quantity_added;
 			// Add back the remaining
-			if( stack.quantity > 0) {
+			if (stack.quantity > 0) {
 				add( stack);
 			}
 		}
 		else {
 			// No available slot, drop
-			// TODO: We should drop on the floor an item we can't store
 		}
 	}
 }
@@ -172,20 +197,32 @@ bool ItemStorage::remove(int item) {
 	return false;
 }
 
-void ItemStorage::sort() {
-	bubbleSort(storage, slot_number);
+int compareItemStack (const void *a, const void *b)
+{
+	const ItemStack *i1 = static_cast<const ItemStack*>(a);
+	const ItemStack *i2 = static_cast<const ItemStack*>(b);
+	if ((*i1) > (*i2))
+		return 1;
+	else
+		return -1;
 }
 
-//TODO: handle stackable items
-bool ItemStorage::full() {
+void ItemStorage::sort() {
+	qsort(storage, slot_number, sizeof(ItemStack), compareItemStack);
+}
+
+bool ItemStorage::full(int item) {
 	for (int i=0; i<slot_number; i++) {
+		if (storage[i].item == item && items->items[item].max_quantity > 1) {
+			return false;
+		}
 		if (storage[i].item == 0) {
 			return false;
 		}
 	}
 	return true;
 }
- 
+
 /**
  * Get the number of the specified item carried (not equipped)
  */
