@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -19,13 +19,15 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class ItemStorage
  */
 
+
+#pragma once
 #ifndef ITEM_STORAGE_H
 #define ITEM_STORAGE_H
 
 #include <SDL.h>
-#include "ItemManager.h"
 
-using namespace std;
+class ItemManager;
+class ItemStack;
 
 class ItemStorage {
 protected:
@@ -38,17 +40,18 @@ public:
 
 	ItemStack & operator [] (int slot);
 
-	void setItems(string s);
-	void setQuantities(string s);
-	string getItems();
-	string getQuantities();
+	void setItems(std::string s);
+	void setQuantities(std::string s);
+	int getSlotNumber();
+	std::string getItems();
+	std::string getQuantities();
 	void add(ItemStack stack, int slot = -1);
 	void substract(int slot, int quantity = 1);
 	bool remove(int item);
 	void sort();
 	void clear();
 
-	bool full();
+	bool full(int item);
 	int count(int item);
 	bool contain(int item);
 
