@@ -1,5 +1,7 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Stefan Beller
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -20,28 +22,36 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  *
  * Handles floor loot
  */
- 
+
+
+#pragma once
 #ifndef LOOT_MANAGER_H
 #define LOOT_MANAGER_H
 
-#include <string>
-#include <sstream>
+#include "Animation.h"
+#include "AnimationSet.h"
+#include "AnimationManager.h"
+
+#include "Loot.h"
+#include "ItemManager.h"
+#include "Settings.h"
+
 #include <SDL.h>
 #include <SDL_image.h>
 #include <SDL_mixer.h>
 
-#include "Utils.h"
-#include "ItemManager.h"
-#include "WidgetTooltip.h"
-#include "EnemyManager.h"
-#include "SharedResources.h"
+#include <string>
 
-struct LootDef {
-	ItemStack stack;
-	int frame;
-	Point pos;
-	int gold;
-	TooltipData tip;
+class EnemyManager;
+class MapRenderer;
+class MenuInventory;
+class WidgetTooltip;
+
+class CurrencyRange {
+public:
+	std::string filename;
+	int low;
+	int high;
 };
 
 
@@ -60,60 +70,54 @@ private:
 
 	ItemManager *items;
 	WidgetTooltip *tip;
-	EnemyManager *enemies;
-	MapIso *map;
+	MapRenderer *map;
+	StatBlock *hero;
 
 	// functions
 	void loadGraphics();
-	void calcTables();
-	int lootLevel(int base_level);
-	
-	SDL_Surface *flying_loot[64];
-	SDL_Surface *flying_gold[3];
-	
-	string animation_id[64];
-	int animation_count;
-	
-	Mix_Chunk *loot_flip;
-	
-	Point frame_size;
-	int frame_count; // the last frame is the "at-rest" floor loot graphic
-	
+
+	SoundManager::SoundID sfx_currency;
+	SoundManager::SoundID sfx_loot;
+
 	// loot refers to ItemManager indices
-	LootDef loot[256]; // TODO: change to dynamic list without limits
-	
-	// loot tables multiplied out
-	// currently loot can range from levels 0-20
-	int loot_table[21][1024]; // level, number.  the int is an item id
-	int loot_table_count[21]; // total number per level
-	
-	// animation vars
-	int anim_loot_frames;
-	int anim_loot_duration;
-	
+	std::vector<Loot> loot;
+
+	SDL_Rect animation_pos;
+	Point animation_offset;
+	std::vector<CurrencyRange> currency_range;
+
+	// enemies which should drop loot, but didnt yet.
+	std::vector<const class Enemy*> enemiesDroppingLoot;
+
 public:
-	LootManager(ItemManager *_items, EnemyManager *_enemies, MapIso *_map);
+	static LootManager *getInstance();
+	LootManager(ItemManager *_items, MapRenderer *_map, StatBlock *_hero);
+	LootManager(const LootManager &copy); // not implemented
 	~LootManager();
 
 	void handleNewMap();
 	void logic();
 	void renderTooltips(Point cam);
 	void checkEnemiesForLoot();
+
+	void playCurrencySound(Point loot_pos = Point(0,0));
+
+	// called by enemy, who definitly wants to drop loot.
+	void addEnemyLoot(const Enemy *e);
 	void checkMapForLoot();
-	bool isFlying(int loot_index);
-	void determineLoot(int base_level, Point pos);
-	int randomItem(int base_level);
+	void determineLootByEnemy(const Enemy *e, Point pos); // pick from enemy-specific loot table
 	void addLoot(ItemStack stack, Point pos);
-	void addGold(int count, Point pos);
-	void removeLoot(int index);
-	ItemStack checkPickup(Point mouse, Point cam, Point hero_pos, int &gold, bool inv_full);
-	
-	Renderable getRender(int index);
-	
-	int tooltip_margin;
-	int loot_count;
+	void addCurrency(int count, Point pos);
+	ItemStack checkPickup(Point mouse, Point cam, Point hero_pos, int &currency, MenuInventory *inv);
+	ItemStack checkAutoPickup(Point hero_pos, int &currency);
+	ItemStack checkNearestPickup(Point hero_pos, int &currency, MenuInventory *inv);
+
+	void addRenders(std::vector<Renderable> &ren, std::vector<Renderable> &ren_dead);
+
+	int tooltip_margin; // pixels between loot drop center and label
 	bool full_msg;
-	
 };
 
+extern LootManager *lootManager;
+
 #endif
