@@ -1,5 +1,6 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -21,12 +22,14 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * The base class for Menu objects
  */
 
+
+#pragma once
 #ifndef MENU_H
 #define MENU_H
 
 #include <SDL_image.h>
-#include "FontEngine.h"
-#include "InputState.h"
+#include <string>
+#include "SoundManager.h"
 
 class Menu {
 protected:
@@ -39,8 +42,13 @@ public:
 
 	bool visible;
 	SDL_Rect window_area;
+	std::string alignment;
 
+	virtual void align();
 	virtual void render() = 0;
+
+	SoundManager::SoundID sfx_open;
+	SoundManager::SoundID sfx_close;
 };
 
 #endif
