@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Igor Paliychuk
 
 This file is part of FLARE.
 
@@ -15,6 +16,8 @@ You should have received a copy of the GNU General Public License along with
 FLARE.  If not, see http://www.gnu.org/licenses/
 */
 
+
+#pragma once
 #ifndef MENU_CONFIRM_H
 #define MENU_CONFIRM_H
 
@@ -22,7 +25,6 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "WidgetButton.h"
 #include <SDL_image.h>
 
-
 class MenuConfirm : public Menu {
 protected:
 	void loadGraphics();
@@ -31,15 +33,18 @@ protected:
 	WidgetButton *buttonClose;
 	WidgetLabel label;
 
-	string boxMsg;
+	std::string boxMsg;
+	bool hasConfirmButton;
 public:
-	MenuConfirm(string, string);
+	MenuConfirm(const std::string&, const std::string&);
 	~MenuConfirm();
 
+	void update();
 	void logic();
 	virtual void render();
 
 	bool confirmClicked;
+	bool cancelClicked;
 };
 
 #endif
