@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Pavel Kirpichyov (Cheshire)
+Copyright © 2011-2012 Pavel Kirpichyov (Cheshire)
 
 This file is part of FLARE.
 
@@ -21,24 +21,26 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Handles the display of the Enemy info of the screen
  */
 
+
+#pragma once
 #ifndef MENU_ENEMY_H
 #define MENU_ENEMY_H
 
-#include "StatBlock.h"
-#include "Utils.h"
-#include "FontEngine.h"
-#include "SharedResources.h"
 #include "Enemy.h"
+#include "Settings.h"
+#include "WidgetLabel.h"
 
 #include <SDL.h>
 #include <SDL_image.h>
 
-const int MENU_ENEMY_TIMEOUT = FRAMES_PER_SEC * 10;
-
-class MenuEnemy {
+class MenuEnemy : public Menu {
 private:
 	SDL_Surface *background;
 	SDL_Surface *bar_hp;
+	SDL_Rect bar_pos;
+	LabelInfo text_pos;
+	bool custom_text_pos;
+	SDL_Color color_normal;
 public:
 	MenuEnemy();
 	~MenuEnemy();
