@@ -1,5 +1,5 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
 
 This file is part of FLARE.
 
@@ -21,29 +21,16 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 
 #include "MenuExit.h"
 #include "SharedResources.h"
-#include "WidgetLabel.h"
-
+#include "Settings.h"
 
 MenuExit::MenuExit() : Menu() {
 
 	exitClicked = false;
 
-	window_area.w = 192;
-	window_area.h = 64;
-	window_area.x = (VIEW_W/2) - (window_area.w/2);
-	window_area.y = (VIEW_H - window_area.h)/2;
-	
 	buttonExit = new WidgetButton(mods->locate("images/menus/buttons/button_default.png"));
 	buttonExit->label = msg->get("Exit");
-	buttonExit->pos.x = VIEW_W_HALF - buttonExit->pos.w/2;
-	buttonExit->pos.y = VIEW_H/2;
-	buttonExit->refresh();
 
 	buttonClose = new WidgetButton(mods->locate("images/menus/buttons/button_x.png"));
-	buttonClose->pos.x = window_area.x + window_area.w;
-	buttonClose->pos.y = window_area.y;
-	
-	label.set(window_area.x + window_area.w/2, window_area.y + 10, JUSTIFY_CENTER, VALIGN_TOP, msg->get("Save and exit to title?"), FONT_WHITE);
 
 	loadGraphics();
 }
@@ -52,19 +39,29 @@ void MenuExit::loadGraphics() {
 	background = IMG_Load(mods->locate("images/menus/confirm_bg.png").c_str());
 	if(!background) {
 		fprintf(stderr, "Couldn't load image: %s\n", IMG_GetError());
-		SDL_Quit();
+	} else {
+		// optimize
+		SDL_Surface *cleanup = background;
+		background = SDL_DisplayFormatAlpha(background);
+		SDL_FreeSurface(cleanup);
 	}
-	
-	// optimize
-	SDL_Surface *cleanup = background;
-	background = SDL_DisplayFormatAlpha(background);
-	SDL_FreeSurface(cleanup);	
+}
+
+void MenuExit::update() {
+	buttonExit->pos.x = VIEW_W_HALF - buttonExit->pos.w/2;
+	buttonExit->pos.y = VIEW_H/2;
+	buttonExit->refresh();
+
+	buttonClose->pos.x = window_area.x + window_area.w;
+	buttonClose->pos.y = window_area.y;
+
+	label.set(window_area.x + window_area.w/2, window_area.y + window_area.h - (buttonExit->pos.h * 2), JUSTIFY_CENTER, VALIGN_TOP, msg->get("Save and exit to title?"), font->getColor("menu_normal"));
 }
 
 void MenuExit::logic() {
 	if (visible) {
 		if (buttonExit->checkClick()) {
-			exitClicked = true;	
+			exitClicked = true;
 		}
 		if (buttonClose->checkClick()) {
 			visible = false;
@@ -73,19 +70,21 @@ void MenuExit::logic() {
 }
 
 void MenuExit::render() {
-	SDL_Rect src;
+	if (visible) {
+		SDL_Rect src;
 
-	// background
-	src.x = 0;
-	src.y = 0;
-	src.w = window_area.w;
-	src.h = window_area.h;
-	SDL_BlitSurface(background, &src, screen, &window_area);
+		// background
+		src.x = 0;
+		src.y = 0;
+		src.w = window_area.w;
+		src.h = window_area.h;
+		SDL_BlitSurface(background, &src, screen, &window_area);
 
-	label.render();
+		label.render();
 
-	buttonExit->render();
-	buttonClose->render();
+		buttonExit->render();
+		buttonClose->render();
+	}
 }
 
 MenuExit::~MenuExit() {
