@@ -1,5 +1,5 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
 
 This file is part of FLARE.
 
@@ -19,12 +19,13 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuExit
  */
 
+
+#pragma once
 #ifndef MENU_EXIT_h
 #define MENU_EXIT_H
 
 #include "Menu.h"
 #include "WidgetButton.h"
-#include "SharedResources.h"
 
 #include <SDL_image.h>
 
@@ -43,6 +44,7 @@ public:
 	MenuExit();
 	~MenuExit();
 
+	void update();
 	void logic();
 	virtual void render();
 
