@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Stefan Beller
 
 This file is part of FLARE.
 
@@ -19,23 +20,21 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuHUDLog
  */
 
+#include "Menu.h"
 #include "MenuHUDLog.h"
 #include "SharedResources.h"
+#include "Settings.h"
 
 using namespace std;
 
 MenuHUDLog::MenuHUDLog() {
-	
-	log_count = 0;
+
 	list_area.x = 224;
 	list_area.y = 416;
+	font->setFont("font_regular");
 	paragraph_spacing = font->getLineHeight()/2;
-	
-	for (int i=0; i<MAX_HUD_MESSAGES; i++) {
-		msg_buffer[i] = NULL;
-		log_msg[i] = "";
-		msg_age[i] = 0;
-	}
+
+	color_normal = font->getColor("menu_normal");
 }
 
 /**
@@ -44,7 +43,7 @@ MenuHUDLog::MenuHUDLog() {
  */
 int MenuHUDLog::calcDuration(const string& s) {
 	// 5 seconds plus an extra second per 10 letters
-	return FRAMES_PER_SEC * 5 + s.length() * (FRAMES_PER_SEC/10);
+	return MAX_FRAMES_PER_SEC * 5 + s.length() * (MAX_FRAMES_PER_SEC/10);
 }
 
 /**
@@ -52,8 +51,11 @@ int MenuHUDLog::calcDuration(const string& s) {
  * Age messages
  */
 void MenuHUDLog::logic() {
-	for (int i=0; i<log_count; i++) {
-		if (msg_age[i] > 0) msg_age[i]--;
+	for (unsigned i=0; i<msg_age.size(); i++) {
+		if (msg_age[i] > 0)
+			msg_age[i]--;
+		else
+			remove(i);
 	}
 }
 
@@ -62,16 +64,16 @@ void MenuHUDLog::logic() {
  * New messages appear on the screen for a brief time
  */
 void MenuHUDLog::render() {
-	
+
 	SDL_Rect dest;
-	dest.x = 32;
-	dest.y = VIEW_H - 40;
-	
+	dest.x = window_area.x;
+	dest.y = window_area.y;
+
 
 	// go through new messages
-	for (int i=log_count-1; i>=0; i--) {
+	for (int i = msg_age.size() - 1; i >= 0; i--) {
 		if (msg_age[i] > 0 && dest.y > 64) {
-		
+
 			dest.y -= msg_buffer[i]->h + paragraph_spacing;
 			SDL_BlitSurface(msg_buffer[i], NULL, screen, &dest);
 		}
@@ -85,56 +87,45 @@ void MenuHUDLog::render() {
  */
 void MenuHUDLog::add(const string& s) {
 
-	if (log_count == MAX_HUD_MESSAGES) {
-		remove(0); // remove the oldest message
-	}
-	
 	// add new message
-	log_msg[log_count] = s;
-	msg_age[log_count] = calcDuration(s);
-	
-	// force HUD messages to vanish in order
-	if (log_count > 0) {
-		if (msg_age[log_count] < msg_age[log_count-1])
-			msg_age[log_count] = msg_age[log_count-1];
-	}
-	
-	// render the log entry and store it in a buffer
-	Point size = font->calc_size(s, list_area.x);
-	msg_buffer[log_count] = createSurface(size.x, size.y);
-	font->renderShadowed(s, 0, 0, JUSTIFY_LEFT, msg_buffer[log_count], list_area.x, FONT_WHITE);
+	log_msg.push_back(s);
+	msg_age.push_back(calcDuration(s));
 
-	log_count++;
+	// force HUD messages to vanish in order
+	if (msg_age.size() > 1) {
+		const int last = msg_age.size()-1;
+		if (msg_age[last] < msg_age[last-1])
+			msg_age[last] = msg_age[last-1];
+	}
+
+	// render the log entry and store it in a buffer
+	font->setFont("font_regular");
+	Point size = font->calc_size(s, window_area.w);
+	msg_buffer.push_back(createAlphaSurface(size.x, size.y));
+	font->renderShadowed(s, 0, 0, JUSTIFY_LEFT, msg_buffer.back(), window_area.w, color_normal);
 }
 
 /**
  * Remove the given message from the list
  */
 void MenuHUDLog::remove(int msg_index) {
-
-	SDL_FreeSurface(msg_buffer[msg_index]);
-	msg_buffer[msg_index] = NULL;
-
-	for (int i=msg_index; i<MAX_HUD_MESSAGES-1; i++) {
-		log_msg[i] = log_msg[i+1];
-		msg_age[i] = msg_age[i+1];
-		msg_buffer[i] = msg_buffer[i+1];
-	}
-
-	log_count--;
+	SDL_FreeSurface(msg_buffer.at(msg_index));
+	msg_buffer.erase(msg_buffer.begin()+msg_index);
+	msg_age.erase(msg_age.begin()+msg_index);
+	log_msg.erase(log_msg.begin()+msg_index);
 }
 
 void MenuHUDLog::clear() {
-	log_count = 0;
-	for (int i=0; i<MAX_HUD_MESSAGES; i++) {
+	for (unsigned i=0; i<msg_buffer.size(); i++) {
 		SDL_FreeSurface(msg_buffer[i]);
-		msg_buffer[i] = NULL;
 	}
-
+	msg_buffer.clear();
+	msg_age.clear();
+	log_msg.clear();
 }
 
 MenuHUDLog::~MenuHUDLog() {
-	for (int i=0; i<MAX_HUD_MESSAGES; i++) {
+	for (unsigned i=0; i<msg_buffer.size(); i++) {
 		SDL_FreeSurface(msg_buffer[i]);
 	}
 }
