@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -19,34 +19,32 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuHUDLog
  */
 
+
+#pragma once
 #ifndef MENU_HUD_LOG_H
 #define MENU_HUD_LOG_H
 
-#include "Settings.h"
 #include "Utils.h"
-#include "FontEngine.h"
 
 #include <SDL.h>
 #include <SDL_image.h>
 
 #include <string>
+#include <vector>
 
-
-const int MAX_HUD_MESSAGES = 16;
-
-class MenuHUDLog{
+class MenuHUDLog : public Menu {
 private:
 
 	int calcDuration(const std::string& s);
 
-	string log_msg[MAX_HUD_MESSAGES];
-	int msg_age[MAX_HUD_MESSAGES];
-	SDL_Surface *msg_buffer[MAX_HUD_MESSAGES];
+	std::vector<std::string> log_msg;
+	std::vector<int> msg_age;
+	std::vector<SDL_Surface*> msg_buffer;
 
-	int log_count;
 	int paragraph_spacing;
-	
-	
+
+	SDL_Color color_normal;
+
 public:
 
 	MenuHUDLog();
@@ -56,7 +54,7 @@ public:
 	void add(const std::string& s);
 	void remove(int msg_index);
 	void clear();
-	
+
 	Point list_area;
 };
 
