@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Igor Paliychuk
 
 This file is part of FLARE.
 
@@ -19,40 +20,29 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuInventory
  */
 
+
+#pragma once
 #ifndef MENU_INVENTORY_H
 #define MENU_INVENTORY_H
 
-#include "InputState.h"
-#include "Utils.h"
-#include "FontEngine.h"
-#include "ItemManager.h"
-#include "WidgetTooltip.h"
-#include "StatBlock.h"
-#include "PowerManager.h"
 #include "MenuItemStorage.h"
-#include "WidgetButton.h"
-#include "SharedResources.h"
+#include "WidgetLabel.h"
 
 #include <SDL.h>
 #include <SDL_image.h>
 
 #include <string>
 
+class InputState;
+class ItemManager;
+class PowerManager;
+class StatBlock;
+class WidgetButton;
+
 const int EQUIPMENT = 0;
 const int CARRIED = 1;
 
-const int MAX_EQUIPPED = 4;
-const int MAX_CARRIED = 64;
-
-enum InventorySlotsType {
-	SLOT_MAIN = 0,
-	SLOT_BODY = 1,
-	SLOT_OFF = 2,
-	SLOT_ARTIFACT = 3
-};
-
-
-class MenuInventory {
+class MenuInventory : public Menu {
 private:
 	ItemManager *items;
 	StatBlock *stats;
@@ -64,10 +54,24 @@ private:
 
 	SDL_Surface *background;
 	WidgetButton *closeButton;
-	
+
+	int MAX_EQUIPPED;
+	int MAX_CARRIED;
+
+	// label and widget positions
+	Point close_pos;
+	LabelInfo title;
+	LabelInfo currency_lbl;
+	SDL_Rect help_pos;
+	int carried_cols;
+	int carried_rows;
+	SDL_Color color_normal;
+	SDL_Color color_high;
+
 public:
 	MenuInventory(ItemManager *items, StatBlock *stats, PowerManager *powers);
 	~MenuInventory();
+	void update();
 	void logic();
 	void render();
 	TooltipData checkTooltip(Point mouse);
@@ -79,32 +83,37 @@ public:
 
 	void add( ItemStack stack, int area = CARRIED, int slot = -1);
 	void remove(int item);
-	void addGold(int count);
-	bool buy(ItemStack stack, Point mouse = Point());
+	void removeEquipped(int item);
+	void addCurrency(int count);
+	bool buy(ItemStack stack, int tab);
 	bool sell(ItemStack stack);
+	bool stashAdd(ItemStack stack);
 
-	bool full();
+	bool full(int item);
 	int getItemCountCarried(int item);
 	bool isItemEquipped(int item);
 	bool requirementsMet(int item);
-	
+
 	void applyEquipment(ItemStack *equipped);
+	void applyItemStats(ItemStack *equipped);
+	void applyItemSetBonuses(ItemStack *equipped);
 
-	bool visible;
+	void clearHighlight();
 
-	SDL_Rect window_area;
 	SDL_Rect carried_area;
-	SDL_Rect equipped_area;
+	std::vector<SDL_Rect> equipped_area;
+	std::vector<std::string> slot_type;
+	std::vector<std::string> slot_desc;
 
 	MenuItemStorage inventory[2];
-	int gold;
+	int currency;
 	int drag_prev_src;
 
 	// the following two are separate because artifacts don't display on the hero.
 	// so we only update the hero sprites when non-artifact changes occur.
 	bool changed_equipment;
 	bool changed_artifact;
-	
+
 	std::string log_msg;
 
 };
