@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -19,95 +20,102 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuManager
  */
 
+
+#pragma once
 #ifndef MENU_MANAGER_H
 #define MENU_MANAGER_H
 
 #include <SDL.h>
 #include <SDL_image.h>
 #include <SDL_mixer.h>
-#include "Utils.h"
-#include "FontEngine.h"
-#include "InputState.h"
-#include "MenuInventory.h"
-#include "MenuPowers.h"
-#include "MenuCharacter.h"
-#include "MenuLog.h"
-#include "MenuHUDLog.h"
-#include "StatBlock.h"
-#include "MenuActionBar.h"
-#include "MenuHPMP.h"
-#include "WidgetTooltip.h"
+
 #include "ItemManager.h"
-#include "PowerManager.h"
-#include "MenuMiniMap.h"
-#include "MenuExperience.h"
-#include "MenuEnemy.h"
-#include "MenuVendor.h"
-#include "MenuTalker.h"
-#include "MenuExit.h"
-#include "CampaignManager.h"
-#include "SharedResources.h"
+#include "WidgetTooltip.h"
+
+class Menu;
+class MenuInventory;
+class MenuPowers;
+class MenuCharacter;
+class MenuLog;
+class MenuHUDLog;
+class MenuActionBar;
+class MenuStatBar;
+class MenuMiniMap;
+class MenuNPCActions;
+class MenuEnemy;
+class MenuVendor;
+class MenuTalker;
+class MenuExit;
+class MenuActiveEffects;
+class MenuStash;
+class CampaignManager;
+class ItemManager;
+class PowerManager;
+class StatBlock;
 
 const int DRAG_SRC_POWERS = 1;
 const int DRAG_SRC_INVENTORY = 2;
 const int DRAG_SRC_ACTIONBAR = 3;
 const int DRAG_SRC_VENDOR = 4;
+const int DRAG_SRC_STASH = 5;
 
 class MenuManager {
 private:
-	
+
 	SDL_Surface *icons;
 
 	PowerManager *powers;
 	StatBlock *stats;
 	CampaignManager *camp;
-	
+
 	TooltipData tip_buf;
 
 	bool key_lock;
-	void loadSounds();
 	void loadIcons();
-	
+
 	bool dragging;
 	ItemStack drag_stack;
 	int drag_power;
 	int drag_src;
 
 	bool done;
-	
+
 public:
-	MenuManager(PowerManager *powers, StatBlock *stats, CampaignManager *camp);
+	MenuManager(PowerManager *powers, StatBlock *stats, CampaignManager *camp, ItemManager *items);
+	MenuManager(const MenuManager &copy); // not implemented
 	~MenuManager();
 	void logic();
 	void render();
 	void renderIcon(int icon_id, int x, int y);
-	void closeAll(bool play_sound);
-	void closeLeft(bool play_sound);
-	void closeRight(bool play_sound);
+	void closeAll();
+	void closeLeft();
+	void closeRight();
 
+	std::vector<Menu*> menus;
+	ItemManager *items;
 	MenuInventory *inv;
 	MenuPowers *pow;
 	MenuCharacter *chr;
 	MenuLog *log;
 	MenuHUDLog *hudlog;
 	MenuActionBar *act;
-	MenuHPMP *hpmp;
+	MenuStatBar *hp;
+	MenuStatBar *mp;
+	MenuStatBar *xp;
 	WidgetTooltip *tip;
 	MenuMiniMap *mini;
-	MenuExperience *xp;
+	MenuNPCActions *npc;
 	MenuEnemy *enemy;
 	MenuVendor *vendor;
 	MenuTalker *talker;
 	MenuExit *exit;
-	ItemManager *items;
-	
+	MenuActiveEffects *effects;
+	MenuStash *stash;
+
 	bool pause;
 	bool menus_open;
-	ItemStack drop_stack;	
+	ItemStack drop_stack;
 
-	Mix_Chunk *sfx_open;
-	Mix_Chunk *sfx_close;
-	
 	bool requestingExit() { return done; }
 };
 
