@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -19,28 +19,44 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuMiniMap
  */
 
+#pragma once
 #ifndef MENU_MINI_MAP_H
 #define MENU_MINI_MAP_H
 
 #include <SDL.h>
 #include "Utils.h"
-#include "MapCollision.h"
+#include "WidgetLabel.h"
 
-class MenuMiniMap {
+class MapCollision;
+class WidgetLabel;
+
+class MenuMiniMap : public Menu {
 private:
 	Uint32 color_wall;
 	Uint32 color_obst;
 	Uint32 color_hero;
-	Point map_center;
-	SDL_Rect map_area;
 
-public: 
+	SDL_Surface *map_surface;
+	Point map_size;
+
+	SDL_Rect pos;
+	LabelInfo text_pos;
+	WidgetLabel *label;
+
+	void createMapSurface();
+	void renderIso(Point hero_pos);
+	void renderOrtho(Point hero_pos);
+	void prerenderOrtho(MapCollision *collider);
+	void prerenderIso(MapCollision *collider);
+
+public:
 	MenuMiniMap();
 	~MenuMiniMap();
 
-	void render(MapCollision *collider, Point hero_pos, int map_w, int map_h);
-	void renderIso(MapCollision *collider, Point hero_pos, int map_w, int map_h);
-
+	void render();
+	void render(Point hero_pos);
+	void prerender(MapCollision *collider, int map_w, int map_h);
+	void getMapTitle(std::string map_title);
 };
 
 
