@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger and morris989
+Copyright © 2011-2012 Clint Bellanger and morris989
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -19,52 +20,69 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class MenuTalker
  */
 
+
+#pragma once
 #ifndef MENU_TALKER_H
 #define MENU_TALKER_H
 
 #include <SDL.h>
 #include <SDL_image.h>
 #include <SDL_mixer.h>
+
 #include "Utils.h"
-#include "FontEngine.h"
-#include "NPC.h"
-#include "CampaignManager.h"
+
 #include <string>
 #include <sstream>
-#include "WidgetButton.h"
 
+class CampaignManager;
+class MenuManager;
+class NPC;
+class WidgetButton;
 
-class MenuTalker {
+class MenuTalker : public Menu {
 private:
 	CampaignManager *camp;
+	MenuManager *menu;
 
 	void loadGraphics();
 	SDL_Surface *background;
 	SDL_Surface *portrait;
 	SDL_Surface *msg_buffer;
-	string hero_name;
+	std::string hero_name;
 
 	int dialog_node;
+	unsigned int event_cursor;
+
+	Point close_pos;
+	Point advance_pos;
+	SDL_Rect dialog_pos;
+	SDL_Rect text_pos;
+	Point text_offset;
+	SDL_Rect portrait_he;
+	SDL_Rect portrait_you;
+
+	std::string font_who;
+	std::string font_dialog;
+
+	SDL_Color color_normal;
 
 public:
-	MenuTalker(CampaignManager *camp);
+	MenuTalker(MenuManager *menu,CampaignManager *camp);
 	~MenuTalker();
 
 	NPC *npc;
-	
-	void chooseDialogNode();
+
+	void chooseDialogNode(int requested_node = -1);
+	void update();
 	void logic();
 	void render();
-	void setHero(string name, string portrait_filename);
+	void setHero(const std::string& name, const std::string& portrait_filename);
 	void createBuffer();
-	
-	bool visible;
-	int event_cursor;
-	bool accept_lock;
+
+	bool vendor_visible;
 
 	WidgetButton *advanceButton;
 	WidgetButton *closeButton;
-	
 };
 
 #endif
