@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Thane Brimhall
+Copyright © 2011-2012 Thane Brimhall
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -21,33 +22,30 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  *
  * The MessageEngine class allows translation of messages in FLARE by comparing them to
  * .po files in a format similar to gettext.
- * 
+ *
  * This class is primarily used for making sure FLARE is flexible and translatable.
  */
 
+
+#pragma once
 #ifndef MESSAGE_ENGINE_H
 #define MESSAGE_ENGINE_H
 
-using namespace std;
-
-#include "GetText.h"
-#include "Settings.h"
 #include <map>
 
 class MessageEngine {
 
 private:
-	map<string,string> messages;
-	string str(int i);
-
+	std::map<std::string,std::string> messages;
+	std::string str(int i);
+	std::string unescape(std::string msg);
 public:
 	MessageEngine();
-	string get(string key);
-	string get(string key, int i);
-	string get(string key, string s);
-	string get(string key, int i, string s);
-	string get(string key, int i, int j);
-
+	std::string get(const std::string& key);
+	std::string get(const std::string& key, int i);
+	std::string get(const std::string& key, const std::string& s);
+	std::string get(const std::string& key, int i, const std::string& s);
+	std::string get(const std::string& key, int i, int j);
 };
 
 #endif
