@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -19,58 +20,54 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class NPC
  */
 
+
+#pragma once
 #ifndef NPC_H
 #define NPC_H
 
 #include "Entity.h"
+#include "ItemStorage.h"
+#include "Utils.h"
+
 #include <SDL.h>
 #include <SDL_image.h>
 #include <SDL_mixer.h>
 #include <string>
-#include "Utils.h"
-#include "ItemManager.h"
-#include "ItemStorage.h"
-#include "MapIso.h"
+#include <vector>
 
-using namespace std;
+class ItemManager;
+class MapRenderer;
 
 const int NPC_VENDOR_MAX_STOCK = 80;
-const int NPC_MAX_VOX = 8;
 const int NPC_VOX_INTRO = 0;
-
-const int NPC_MAX_DIALOG = 32;
-const int NPC_MAX_EVENTS = 16;
+const int NPC_VOX_QUEST = 1;
+const int NPC_NO_DIALOG_AVAIL = -1;
 
 class NPC : public Entity {
 protected:
 	ItemManager *items;
 
-	// animation info
-	SDL_Surface *sprites;
-	int anim_frames;
-	int anim_duration;
-	int current_frame;
-	
 public:
-	NPC(MapIso *_map, ItemManager *_items);
+	NPC(MapRenderer *_map, ItemManager *_items);
 	~NPC();
-	void load(string npc_id);
-	void loadGraphics(string filename_sprites, string filename_portrait);
-	void loadSound(string filename, int type);
+	void load(const std::string& npc_id, int hero_level);
+	void loadGraphics(const std::string& filename_portrait);
+	int loadSound(const std::string& fname, int type);
 	void logic();
-	bool playSound(int type);
-	int chooseDialogNode();
-	bool processDialog(int dialog_node, int &event_cursor);
+	bool playSound(int type, int id=-1);
+	void getDialogNodes(std::vector<int> &result);
+	std::string getDialogTopic(unsigned int dialog_node);
+	bool processDialog(unsigned int dialog_node, unsigned int& event_cursor);
 	virtual Renderable getRender();
-	
+
 	// general info
-	string name;
+	std::string name;
+	std::string filename;
+	std::string gfx; // filename of sprite.
 	Point pos; // map position
 	int level; // used in determining item quality
-	
-	// public animation info
-	Point render_size;
-	Point render_offset;
+
+	int direction;
 
 	// talker info
 	SDL_Surface *portrait;
@@ -80,16 +77,15 @@ public:
 	bool vendor;
 	ItemStorage stock;
 	int stock_count;
-	int random_stock;
-	
+
 	// vocals
-	Mix_Chunk *vox_intro[NPC_MAX_VOX];
-	int vox_intro_count;
-	
+	std::vector<SoundManager::SoundID> vox_intro;
+	std::vector<SoundManager::SoundID> vox_quests;
+
 	// story and dialog options
-	Event_Component dialog[NPC_MAX_DIALOG][NPC_MAX_EVENTS];
-	int dialog_count;
-	
+	// outer vector is addressing the dialog and the inner vector is
+	// addressing the events during one dialog
+	std::vector<std::vector<Event_Component> > dialog;
 };
 
 #endif
