@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2012 Stefan Beller
 
 This file is part of FLARE.
 
@@ -22,21 +23,33 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  */
 
 #include "QuestLog.h"
-#include <fstream>
-#include "FileParser.h"
+#include "CampaignManager.h"
+#include "Menu.h"
+#include "MenuLog.h"
 #include "SharedResources.h"
+#include "Settings.h"
+#include "FileParser.h"
 #include "UtilsFileSystem.h"
+#include "UtilsParsing.h"
+
+#include <fstream>
+
+using namespace std;
+
 
 QuestLog::QuestLog(CampaignManager *_camp, MenuLog *_log) {
 	camp = _camp;
 	log = _log;
-	
-    newQuestNotification = false;
-    resetQuestNotification = false;
-	quest_count = 0;
+
+	newQuestNotification = false;
+	resetQuestNotification = false;
 	loadAll();
 }
 
+QuestLog::~QuestLog()
+{
+}
+
 /**
  * Load each [mod]/quests/index.txt file
  */
@@ -65,14 +78,14 @@ void QuestLog::loadAll() {
 void QuestLog::loadIndex(const std::string& filename) {
 	ifstream infile;
 	string line;
-	
+
 	infile.open(filename.c_str(), ios::in);
-	
+
 	if (infile.is_open()) {
-		while (!infile.eof()) {
+		while (infile.good()) {
 			line = getLine(infile);
 			if (line.length() > 0) {
-			
+
 				// each line contains a quest file name
 				load(line);
 			}
@@ -88,31 +101,28 @@ void QuestLog::loadIndex(const std::string& filename) {
  * @param filename The quest file name and extension, no path
  */
 void QuestLog::load(const std::string& filename) {
-
 	FileParser infile;
-	int event_count = 0;
-	
-	if (infile.open(mods->locate("quests/" + filename))) {
-		while (infile.next()) {
-			if (infile.new_section) {
-				if (infile.section == "quest") {
-					quest_count++;
-					event_count = 0;
-				}
-			}
-			
-			quests[quest_count-1][event_count].type = infile.key;
-			quests[quest_count-1][event_count].s = msg->get(infile.val);
-			event_count++;
-			
-		}
-		infile.close();
+	if (!infile.open(mods->locate("quests/" + filename))) {
+		fprintf(stderr, "Unable to open quests/%s!\n", filename.c_str());
+		return;
 	}
+
+	while (infile.next()) {
+		if (infile.new_section) {
+			if (infile.section == "quest")
+				quests.push_back(vector<Event_Component>());
+		}
+		Event_Component ev;
+		ev.type = infile.key;
+		ev.s = msg->get(infile.val);
+		quests.back().push_back(ev);
+	}
+	infile.close();
 }
 
 void QuestLog::logic() {
 	if (camp->quest_update) {
-        resetQuestNotification = true;
+		resetQuestNotification = true;
 		camp->quest_update = false;
 		createQuestList();
 	}
@@ -124,9 +134,9 @@ void QuestLog::logic() {
 void QuestLog::createQuestList() {
 	log->clear(LOG_TYPE_QUESTS);
 
-	for (int i=0; i<quest_count; i++) {
-		for (int j=0; j<MAX_QUEST_EVENTS; j++) {
-			
+	for (unsigned int i=0; i<quests.size(); i++) {
+		for (unsigned int j=0; j<quests[i].size(); j++) {
+
 			// check requirements
 			// break (skip to next dialog node) if any requirement fails
 			// if we reach an event that is not a requirement, succeed
@@ -139,7 +149,7 @@ void QuestLog::createQuestList() {
 			}
 			else if (quests[i][j].type == "quest_text") {
 				log->add(quests[i][j].s, LOG_TYPE_QUESTS);
-                newQuestNotification = true;
+				newQuestNotification = true;
 				break;
 			}
 			else if (quests[i][j].type == "") {
@@ -147,5 +157,4 @@ void QuestLog::createQuestList() {
 			}
 		}
 	}
-	
 }
