@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -21,27 +21,30 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Helper text to remind the player of active quests
  */
 
+
+#pragma once
 #ifndef QUEST_LOG_H
 #define QUEST_LOG_H
 
 #include <string>
+#include <vector>
 #include "Utils.h"
-#include "CampaignManager.h"
-#include "MenuLog.h"
 
-const int MAX_QUESTS = 1024;
-const int MAX_QUEST_EVENTS = 8;
+class CampaignManager;
+class MenuLog;
 
 class QuestLog {
 private:
 	CampaignManager *camp;
 	MenuLog *log;
-	
-	Event_Component quests[MAX_QUESTS][MAX_QUEST_EVENTS];
-	int quest_count;
-	
+
+	// inner vector is a chain of events per quest, outer vector is a
+	// list of quests.
+	std::vector<std::vector<Event_Component> >quests;
+
 public:
 	QuestLog(CampaignManager *_camp, MenuLog *_log);
+	~QuestLog();
 	void loadAll();
 	void loadIndex(const std::string& filename);
 	void load(const std::string& filename);
