@@ -1,5 +1,6 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
+Copyright © 2013 Henrik Andersson
 
 This file is part of FLARE.
 
@@ -24,19 +25,32 @@ Generic objects only. Game-specific objects don't belong here.
 Created and destroyed by main.cpp
 **/
 
+
+#pragma once
 #ifndef SHARED_RESOURCES_H
 #define SHARED_RESOURCES_H
 
 #include <SDL.h>
-#include "ModManager.h"
-#include "MessageEngine.h"
-#include "InputState.h"
+
+#include "AnimationManager.h"
+#include "CombatText.h"
 #include "FontEngine.h"
+#include "ImageManager.h"
+#include "InputState.h"
+#include "MessageEngine.h"
+#include "ModManager.h"
+#include "SoundManager.h"
 
 extern SDL_Surface *screen;
-extern ModManager *mods;
-extern MessageEngine *msg;
-extern InputState *inp;
+extern SDL_Joystick *joy;
+
+extern AnimationManager *anim;
+extern CombatText *comb;
 extern FontEngine *font;
+extern ImageManager *imag;
+extern InputState *inpt;
+extern MessageEngine *msg;
+extern ModManager *mods;
+extern SoundManager *snd;
 
 #endif
