@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -21,6 +21,8 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * Various file system function wrappers. Abstracted here to hide OS-specific implementations
  */
 
+
+#pragma once
 #ifndef UTILS_FILE_SYSTEM_H
 #define UTILS_FILE_SYSTEM_H
 
@@ -31,6 +33,7 @@ bool dirExists(std::string path);
 void createDir(std::string path);
 bool fileExists(std::string filename);
 int getFileList(std::string dir, std::string ext, std::vector<std::string> &files);
+int getDirList(std::string dir, std::vector<std::string> &dirs);
 
 
 
