@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -19,46 +19,60 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class WidgetButton
  */
 
+
+#pragma once
 #ifndef WIDGET_BUTTON_H
 #define WIDGET_BUTTON_H
 
+#include "Widget.h"
+#include "WidgetLabel.h"
+#include "WidgetTooltip.h"
+
 #include <SDL.h>
 #include <SDL_image.h>
 #include <SDL_mixer.h>
-#include "Utils.h"
-#include "FontEngine.h"
-#include "InputState.h"
+
 #include <string>
-#include "WidgetLabel.h"
+
 
 const int BUTTON_GFX_NORMAL = 0;
 const int BUTTON_GFX_PRESSED = 1;
 const int BUTTON_GFX_HOVER = 2;
 const int BUTTON_GFX_DISABLED = 3;
 
-class WidgetButton {
+class WidgetButton : public Widget {
 private:
 
 	std::string fileName; // the path to the buttons background image
 
 	SDL_Surface *buttons;
 	Mix_Chunk *click;
-	
+
 	WidgetLabel wlabel;
-	
+
+	SDL_Color color_normal;
+	SDL_Color color_disabled;
+
+	TooltipData tip_buf;
+	TooltipData tip_new;
+	WidgetTooltip *tip;
+	TooltipData checkTooltip(Point mouse);
+
 public:
 	WidgetButton(const std::string& _fileName);
 	~WidgetButton();
 
 	void loadArt();
 	bool checkClick();
-	void render();
+	bool checkClick(int x, int y);
+	void render(SDL_Surface *target = NULL);
 	void refresh();
 
-	string label;
-	SDL_Rect pos;
+	std::string label;
+	std::string tooltip;
 	bool enabled;
 	bool pressed;
+	bool hover;
 };
 
 #endif
