@@ -1,5 +1,6 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
+Copyright © 2012 Stefan Beller
 
 This file is part of FLARE.
 
@@ -17,22 +18,31 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 
 #include "WidgetInput.h"
 #include "SharedResources.h"
+#include "Settings.h"
+#include "SDL_gfxBlitFunc.h"
+
+using namespace std;
+
 
 WidgetInput::WidgetInput() {
-	
+
 	enabled = true;
 	inFocus = false;
 	pressed = false;
+	hover = false;
 	max_characters = 20;
-	
+
 	loadGraphics(mods->locate("images/menus/input.png"));
 
 	// position
 	pos.w = background->w;
 	pos.h = background->h/2;
-	
+
 	cursor_frame = 0;
-	
+
+	render_to_alpha = false;
+
+	color_normal = font->getColor("widget_normal");
 }
 
 void WidgetInput::loadGraphics(const string& filename) {
@@ -43,8 +53,9 @@ void WidgetInput::loadGraphics(const string& filename) {
 	if(!background) {
 		fprintf(stderr, "Couldn't load image: %s\n", IMG_GetError());
 		SDL_Quit();
+		exit(1);
 	}
-	
+
 	// optimize
 	SDL_Surface *cleanup = background;
 	background = SDL_DisplayFormatAlpha(background);
@@ -53,31 +64,44 @@ void WidgetInput::loadGraphics(const string& filename) {
 }
 
 void WidgetInput::logic() {
+	if (logic(inpt->mouse.x,inpt->mouse.y))
+		return;
+}
+
+bool WidgetInput::logic(int x, int y) {
+	Point mouse(x, y);
+
+	// Change the hover state
+	if (isWithin(pos, mouse)) {
+		hover = true;
+	} else {
+		hover = false;
+	}
 
 	if (checkClick()) {
 		inFocus = true;
 	}
 
 	// if clicking elsewhere unfocus the text box
-	if (inp->pressing[MAIN1]) {
-		if (!isWithin(pos, inp->mouse)) {
+	if (inpt->pressing[MAIN1]) {
+		if (!isWithin(pos, inpt->mouse)) {
 			inFocus = false;
 		}
 	}
 
 	if (inFocus) {
 
-		if (inp->inkeys != "") {
+		if (inpt->inkeys != "") {
 			// handle text input
-			text += inp->inkeys;
+			text += inpt->inkeys;
 			if (text.length() > max_characters) {
 				text = text.substr(0, max_characters);
 			}
 		}
-			
+
 		// handle backspaces
-		if (!inp->lock[DEL] && inp->pressing[DEL]) {
-			inp->lock[DEL] = true;
+		if (!inpt->lock[DEL] && inpt->pressing[DEL]) {
+			inpt->lock[DEL] = true;
 			// remove utf-8 character
 			int n = text.length()-1;
 			while (n > 0 && ((text[n] & 0xc0) == 0x80) ) n--;
@@ -87,13 +111,17 @@ void WidgetInput::logic() {
 		// animate cursor
 		// cursor visible one second, invisible the next
 		cursor_frame++;
-		if (cursor_frame == FRAMES_PER_SEC+FRAMES_PER_SEC) cursor_frame = 0;
-		
-	}
+		if (cursor_frame == MAX_FRAMES_PER_SEC+MAX_FRAMES_PER_SEC) cursor_frame = 0;
 
+	}
+	return true;
 }
 
-void WidgetInput::render() {
+void WidgetInput::render(SDL_Surface *target) {
+	if (target == NULL) {
+		target = screen;
+	}
+
 	SDL_Rect src;
 	src.x = 0;
 	src.y = 0;
@@ -102,22 +130,25 @@ void WidgetInput::render() {
 
 	if (!inFocus)
 		src.y = 0;
-	else if (isWithin(pos, inp->mouse))
-		src.y = pos.h;
 	else
 		src.y = pos.h;
 
-	SDL_BlitSurface(background, &src, screen, &pos);
+	if (render_to_alpha)
+		SDL_gfxBlitRGBA(background, &src, target, &pos);
+	else
+		SDL_BlitSurface(background, &src, target, &pos);
+
+	font->setFont("font_regular");
 
 	if (!inFocus) {
-		font->render(text, font_pos.x, font_pos.y, JUSTIFY_LEFT, screen, FONT_WHITE);
+		font->render(text, font_pos.x, font_pos.y, JUSTIFY_LEFT, target, color_normal);
 	}
 	else {
-		if (cursor_frame < FRAMES_PER_SEC) {
-			font->renderShadowed(text + "|", font_pos.x, font_pos.y, JUSTIFY_LEFT, screen, FONT_WHITE);
+		if (cursor_frame < MAX_FRAMES_PER_SEC) {
+			font->renderShadowed(text + "|", font_pos.x, font_pos.y, JUSTIFY_LEFT, target, color_normal);
 		}
 		else {
-			font->renderShadowed(text, font_pos.x, font_pos.y, JUSTIFY_LEFT, screen, FONT_WHITE);		
+			font->renderShadowed(text, font_pos.x, font_pos.y, JUSTIFY_LEFT, target, color_normal);
 		}
 	}
 }
@@ -125,7 +156,8 @@ void WidgetInput::render() {
 void WidgetInput::setPosition(int x, int y) {
 	pos.x = x;
 	pos.y = y;
-	
+
+	font->setFont("font_regular");
 	font_pos.x = pos.x  + (font->getFontHeight()/2);
 	font_pos.y = pos.y + (pos.h/2) - (font->getFontHeight()/2);
 }
@@ -136,26 +168,26 @@ bool WidgetInput::checkClick() {
 	if (!enabled) return false;
 
 	// main button already in use, new click not allowed
-	if (inp->lock[MAIN1]) return false;
+	if (inpt->lock[MAIN1]) return false;
 
 	// main click released, so the button state goes back to unpressed
-	if (pressed && !inp->lock[MAIN1]) {
+	if (pressed && !inpt->lock[MAIN1]) {
 		pressed = false;
-		
-		if (isWithin(pos, inp->mouse)) {
-		
+
+		if (isWithin(pos, inpt->mouse)) {
+
 			// activate upon release
 			return true;
 		}
 	}
 
 	pressed = false;
-	
+
 	// detect new click
-	if (inp->pressing[MAIN1]) {
-		if (isWithin(pos, inp->mouse)) {
-		
-			inp->lock[MAIN1] = true;
+	if (inpt->pressing[MAIN1]) {
+		if (isWithin(pos, inpt->mouse)) {
+
+			inpt->lock[MAIN1] = true;
 			pressed = true;
 
 		}
@@ -163,3 +195,7 @@ bool WidgetInput::checkClick() {
 	return false;
 }
 
+WidgetInput::~WidgetInput() {
+	SDL_FreeSurface(background);
+}
+
