@@ -1,5 +1,5 @@
 /*
-Copyright 2011 kitano
+Copyright © 2011-2012 kitano
 
 This file is part of FLARE.
 
@@ -22,17 +22,17 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * It has two images - one for focused and one for unfocused.
  */
 
+
+#pragma once
 #ifndef WIDGETINPUT_H
 #define WIDGETINPUT_H
 
-#include "FontEngine.h"
-#include "InputState.h"
+#include "Utils.h"
+#include "Widget.h"
 #include <SDL.h>
 #include <string>
 
-using namespace std;
-
-class WidgetInput {
+class WidgetInput : public Widget {
 
 protected:
 
@@ -41,25 +41,30 @@ protected:
 	SDL_Surface *background;
 
 	bool enabled;
-	bool inFocus;
 	bool pressed;
+	bool hover;
 
-	string text; // the text that has been typed into the box
+	std::string text; // the text that has been typed into the box
 	unsigned int max_characters;
 	int cursor_frame;
-	
+
 	Point font_pos;
 
+	SDL_Color color_normal;
+
 public:
 	WidgetInput();
-	
+	~WidgetInput();
+
 	void logic();
-	void render();
+	bool logic(int x, int y);
+	void render(SDL_Surface *target = NULL);
 	bool checkClick();
-	string getText() { return text; }
+	std::string getText() { return text; }
+	void setText(std::string _text) { text = _text; }
 	void setPosition(int x, int y);
 
-	SDL_Rect pos;
+	bool inFocus;
 };
 
 #endif
