@@ -1,5 +1,5 @@
 /*
-Copyright 2011 Clint Bellanger
+Copyright © 2011-2012 Clint Bellanger
 
 This file is part of FLARE.
 
@@ -22,41 +22,62 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * This is preferred to directly displaying text because it helps handle positioning
  */
 
+
+#pragma once
 #ifndef WIDGET_LABEL_H
 #define WIDGET_LABEL_H
 
+#include "Widget.h"
 #include <SDL.h>
-#include "FontEngine.h"
 #include <string>
 
 const int VALIGN_CENTER = 0;
 const int VALIGN_TOP = 1;
 const int VALIGN_BOTTOM = 2;
 
-class WidgetLabel {
+class LabelInfo {
+public:
+	int x,y;
+	int justify,valign;
+	bool hidden;
+	std::string font_style;
+
+	LabelInfo();
+};
+
+LabelInfo eatLabelInfo(std::string val);
+
+class WidgetLabel : public Widget {
 private:
 
 	void refresh();
 	void applyOffsets();
 
-	string text;
-	int color;
+	std::string text;
+	SDL_Color color;
 	int x_origin;
 	int y_origin;
 	int justify;
 	int valign;
+	std::string font_style;
 	SDL_Surface *text_buffer;
 
 public:
 	WidgetLabel();
 	~WidgetLabel();
-	void render();
-	void set(int _x, int _y, int _justify, int _valign, string _text, int _color);
-	
-	void set(string _text);
-	string get() {return text;}
+	void render(SDL_Surface *target = NULL);
+	void set(int _x, int _y, int _justify, int _valign, const std::string& _text, SDL_Color _color);
+	void set(int _x, int _y, int _justify, int _valign, const std::string& _text, SDL_Color _color, std::string _font);
+	void setX(int _x);
+	void setY(int _y);
+	int getX();
+	int getY();
+	void setJustify(int _justify);
 
-	SDL_Rect bounds;	
+	void set(const std::string& _text);
+	std::string get() {return text;}
+
+	SDL_Rect bounds;
 };
 
 #endif
